@@ -60,6 +60,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #define DATE_FORMAT "%Y-%m-%d %T"
 
+#ifndef MYSQL_PORT
+# ifdef MARIADB_PORT
+#  define MYSQL_PORT MARIADB_PORT
+# else
+#  define MYSQL_PORT 3306
+# endif
+#endif
+
 AST_THREADSTORAGE(sql1_buf);
 AST_THREADSTORAGE(sql2_buf);
 AST_THREADSTORAGE(escape_buf);
@@ -172,7 +180,9 @@ static int mysql_log(struct ast_cdr *cdr)
 {
 	struct ast_str *sql1 = ast_str_thread_get(&sql1_buf, 1024), *sql2 = ast_str_thread_get(&sql2_buf, 1024);
 	int retries = 5;
-#if MYSQL_VERSION_ID >= 50013
+#ifdef HAVE_MYSQLCLIENT_BOOL
+	bool my_bool_true = 1;
+#elif HAVE_MYSQLCLIENT_MY_BOOL
 	my_bool my_bool_true = 1;
 #endif
 
@@ -381,6 +391,13 @@ static int my_unload_module(int reload)
 {
 	struct column *entry;
 
+	if (!reload) {
+		if (ast_cdr_unregister(name)) {
+			/* If we can't unregister the backend, we can't unload the module */
+			return -1;
+		}
+	}
+
 	ast_cli_unregister_multiple(cdr_mysql_status_cli, sizeof(cdr_mysql_status_cli) / sizeof(struct ast_cli_entry));
 
 	if (connected) {
@@ -405,7 +422,8 @@ static int my_unload_module(int reload)
 	if (reload) {
 		return ast_cdr_backend_suspend(name);
 	} else {
-		return ast_cdr_unregister(name);
+		/* We unregistered earlier */
+		return 0;
 	}
 }
 
@@ -462,7 +480,9 @@ static int my_load_module(int reload)
 	MYSQL_ROW row;
 	MYSQL_RES *result;
 	char sqldesc[128];
-#if MYSQL_VERSION_ID >= 50013
+#ifdef HAVE_MYSQLCLIENT_BOOL
+	bool my_bool_true = 1;
+#elif HAVE_MYSQLCLIENT_MY_BOOL
 	my_bool my_bool_true = 1;
 #endif
 
