@@ -1,15 +1,15 @@
 /*
  * Copyright (C) 2004-2005 by Objective Systems, Inc.
  *
- * This software is furnished under an open source license and may be 
- * used and copied only in accordance with the terms of this license. 
- * The text of the license may generally be found in the root 
- * directory of this installation in the COPYING file.  It 
+ * This software is furnished under an open source license and may be
+ * used and copied only in accordance with the terms of this license.
+ * The text of the license may generally be found in the root
+ * directory of this installation in the COPYING file.  It
  * can also be viewed online at the following URL:
  *
  *   http://www.obj-sys.com/open/license.html
  *
- * Any redistributions of this file including modified versions must 
+ * Any redistributions of this file including modified versions must
  * maintain this copyright notice.
  *
  *****************************************************************************/
@@ -65,6 +65,8 @@
 #include "asterisk/format_cap.h"
 #include "asterisk/udptl.h"
 #include "asterisk/stasis_channels.h"
+#include "asterisk/format_cache.h"
+#include "asterisk/paths.h"
 
 #include "ootypes.h"
 #include "ooUtils.h"
@@ -85,7 +87,7 @@ struct ooh323_peer;
 /* Helper functions */
 struct ooh323_user *find_user(const char * name, const char *ip);
 struct ooh323_peer *find_peer(const char * name, int port);
-void ooh323_delete_peer(struct ooh323_peer *peer);   
+void ooh323_delete_peer(struct ooh323_peer *peer);
 
 int delete_users(void);
 int delete_peers(void);
@@ -95,7 +97,7 @@ int reload_config(int reload);
 int restart_monitor(void);
 
 int configure_local_rtp(struct ooh323_pvt *p, ooCallData* call);
-void setup_rtp_connection(ooCallData *call, const char *remoteIp, 
+void setup_rtp_connection(ooCallData *call, const char *remoteIp,
                           int remotePort);
 void close_rtp_connection(ooCallData *call);
 struct ast_frame *ooh323_rtp_read
@@ -104,9 +106,6 @@ struct ast_frame *ooh323_rtp_read
 void ooh323_set_write_format(ooCallData *call, struct ast_format *fmt, int txframes);
 void ooh323_set_read_format(ooCallData *call, struct ast_format *fmt);
 
-int ooh323_update_capPrefsOrderForCall
-   (ooCallData *call, struct ast_codec_pref *prefs);
-
 int ooh323_convertAsteriskCapToH323Cap(struct ast_format *format);
 
 int ooh323_convert_hangupcause_asteriskToH323(int cause);
