@@ -590,7 +590,7 @@ static struct adsi_subscript *getsubbyname(struct adsi_script *state, char *name
 	}
 
 	if (state->numsubs > 127) {
-		ast_log(LOG_WARNING, "No more subscript space at line %d of %s\n", lineno, script);
+		ast_log(LOG_WARNING, "No more subscript space at line %d of %s\n", lineno, S_OR(script, "unknown"));
 		return NULL;
 	}
 
@@ -1615,5 +1615,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Asterisk ADSI Programmin
 		.support_level = AST_MODULE_SUPPORT_EXTENDED,
 		.load = load_module,
 		.unload = unload_module,
-		.nonoptreq = "res_adsi",
 		);
