@@ -21,7 +21,7 @@
  * \brief Execute an ISDN RAS
  *
  * \author Mark Spencer <markster@digium.com>
- * 
+ *
  * \ingroup applications
  */
 
@@ -32,7 +32,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 419592 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include <sys/ioctl.h>
 #include <sys/wait.h>
@@ -120,8 +120,10 @@ static pid_t spawn_ras(struct ast_channel *chan, char *args)
 		c = strsep(&stringp, ",");
 	}
 
-	argv[argc++] = "plugin";
-	argv[argc++] = "dahdi.so";
+	if (geteuid() == 0) {
+		argv[argc++] = "plugin";
+		argv[argc++] = "dahdi.so";
+	}
 	argv[argc++] = "stdin";
 
 	/* Finally launch PPP */
@@ -138,7 +140,7 @@ static void run_ras(struct ast_channel *chan, char *args)
 	int signalled = 0;
 	struct dahdi_bufferinfo savebi;
 	int x;
-	
+
 	res = ioctl(ast_channel_fd(chan, 0), DAHDI_GET_BUFINFO, &savebi);
 	if(res) {
 		ast_log(LOG_WARNING, "Unable to check buffer policy on channel %s\n", ast_channel_name(chan));
@@ -168,7 +170,7 @@ static void run_ras(struct ast_channel *chan, char *args)
 			if (WIFEXITED(status)) {
 				ast_verb(3, "RAS on %s terminated with status %d\n", ast_channel_name(chan), WEXITSTATUS(status));
 			} else if (WIFSIGNALED(status)) {
-				ast_verb(3, "RAS on %s terminated with signal %d\n", 
+				ast_verb(3, "RAS on %s terminated with signal %d\n",
 					 ast_channel_name(chan), WTERMSIG(status));
 			} else {
 				ast_verb(3, "RAS on %s terminated weirdly.\n", ast_channel_name(chan));
@@ -194,11 +196,11 @@ static int dahdiras_exec(struct ast_channel *chan, const char *data)
 	char *args;
 	struct dahdi_params dahdip;
 
-	if (!data) 
+	if (!data)
 		data = "";
 
 	args = ast_strdupa(data);
-	
+
 	/* Answer the channel if it's not up */
 	if (ast_channel_state(chan) != AST_STATE_UP)
 		ast_answer(chan);
@@ -223,16 +225,14 @@ static int dahdiras_exec(struct ast_channel *chan, const char *data)
 	return res;
 }
 
-static int unload_module(void) 
+static int unload_module(void)
 {
 	return ast_unregister_application(app);
 }
 
 static int load_module(void)
 {
-	return ((ast_register_application_xml(app, dahdiras_exec)) ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS);
+	return ((ast_register_application_xml(app, dahdiras_exec)) ? AST_MODULE_LOAD_DECLINE : AST_MODULE_LOAD_SUCCESS);
 }
 
 AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "DAHDI ISDN Remote Access Server");
-
-
