@@ -65,6 +65,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<option name="n">
 						<para>Read digits even if the channel is not answered.</para>
 					</option>
+					<option name="p">
+						<para>The extension entered will be considered complete when a <literal>#</literal>
+						is entered.</para>
+					</option>
 				</optionlist>
 			</parameter>
 			<parameter name="timeout">
@@ -102,12 +106,14 @@ enum readexten_option_flags {
 	OPT_SKIP = (1 << 0),
 	OPT_INDICATION = (1 << 1),
 	OPT_NOANSWER = (1 << 2),
+	OPT_POUND_TO_END = (1 << 3),
 };
 
 AST_APP_OPTIONS(readexten_app_options, {
 	AST_APP_OPTION('s', OPT_SKIP),
 	AST_APP_OPTION('i', OPT_INDICATION),
 	AST_APP_OPTION('n', OPT_NOANSWER),
+	AST_APP_OPTION('p', OPT_POUND_TO_END),
 });
 
 static char *app = "ReadExten";
@@ -166,8 +172,7 @@ static int readexten_exec(struct ast_channel *chan, const char *data)
 	if (timeout <= 0)
 		timeout = ast_channel_pbx(chan) ? ast_channel_pbx(chan)->rtimeoutms : 10000;
 
-	if (digit_timeout <= 0)
-		digit_timeout = ast_channel_pbx(chan) ? ast_channel_pbx(chan)->dtimeoutms : 5000;
+	digit_timeout = ast_channel_pbx(chan) ? ast_channel_pbx(chan)->dtimeoutms : 5000;
 
 	if (ast_test_flag(&flags, OPT_INDICATION) && !ast_strlen_zero(arglist.filename)) {
 		ts = ast_get_indication_tone(ast_channel_zone(chan), arglist.filename);
@@ -228,6 +233,11 @@ static int readexten_exec(struct ast_channel *chan, const char *data)
 				break;
 			}
 
+			if (ast_test_flag(&flags, OPT_POUND_TO_END) && res == '#') {
+				exten[x] = 0;
+				break;
+			}
+
 			exten[x] = res;
 			if (!ast_matchmore_extension(chan, arglist.context, exten, 1 /* priority */,
 				S_COR(ast_channel_caller(chan)->id.number.valid, ast_channel_caller(chan)->id.number.str, NULL))) {
