@@ -28,7 +28,7 @@
 /*** MODULEINFO
 	<support_level>core</support_level>
  ***/
- 
+
 #include "asterisk.h"
 
 ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
@@ -40,6 +40,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/channel.h"
 #include "asterisk/dsp.h"	/* use dsp routines for silence detection */
 #include "asterisk/format_cache.h"
+#include "asterisk/paths.h"
 
 /*** DOCUMENTATION
 	<application name="Record" language="en_US">
@@ -81,6 +82,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<option name="t">
 						<para>use alternate '*' terminator key (DTMF) instead of default '#'</para>
 					</option>
+					<option name="u">
+						<para>Don't truncate recorded silence.</para>
+					</option>
 					<option name="x">
 						<para>Ignore all terminator keys (DTMF) and keep recording until hangup.</para>
 					</option>
@@ -101,7 +105,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			If the user hangs up during a recording, all data will be lost and the application will terminate.</para>
 			<variablelist>
 				<variable name="RECORDED_FILE">
-					<para>Will be set to the final filename of the recording.</para>
+					<para>Will be set to the final filename of the recording, without an extension.</para>
 				</variable>
 				<variable name="RECORD_STATUS">
 					<para>This is the final status of the command</para>
@@ -130,9 +134,15 @@ enum {
 	OPTION_STAR_TERMINATE = (1 << 4),
 	OPTION_IGNORE_TERMINATE = (1 << 5),
 	OPTION_KEEP = (1 << 6),
-	FLAG_HAS_PERCENT = (1 << 7),
-	OPTION_ANY_TERMINATE = (1 << 8),
-	OPTION_OPERATOR_EXIT = (1 << 9),
+	OPTION_ANY_TERMINATE = (1 << 7),
+	OPTION_OPERATOR_EXIT = (1 << 8),
+	OPTION_NO_TRUNCATE = (1 << 9),
+};
+
+enum dtmf_response {
+	RESPONSE_NO_MATCH = 0,
+	RESPONSE_OPERATOR,
+	RESPONSE_DTMF,
 };
 
 AST_APP_OPTIONS(app_opts,{
@@ -143,6 +153,7 @@ AST_APP_OPTIONS(app_opts,{
 	AST_APP_OPTION('q', OPTION_QUIET),
 	AST_APP_OPTION('s', OPTION_SKIP),
 	AST_APP_OPTION('t', OPTION_STAR_TERMINATE),
+	AST_APP_OPTION('u', OPTION_NO_TRUNCATE),
 	AST_APP_OPTION('y', OPTION_ANY_TERMINATE),
 	AST_APP_OPTION('x', OPTION_IGNORE_TERMINATE),
 });
@@ -157,43 +168,75 @@ AST_APP_OPTIONS(app_opts,{
  * \param dtmf_integer the integer value of the DTMF key received
  * \param terminator key currently set to be pressed for normal termination
  *
- * \retval 0 do not exit
- * \retval -1 do exit
+ * \returns One of enum dtmf_response
  */
-static int record_dtmf_response(struct ast_channel *chan, struct ast_flags *flags, int dtmf_integer, int terminator)
+static enum dtmf_response record_dtmf_response(struct ast_channel *chan,
+	struct ast_flags *flags, int dtmf_integer, int terminator)
 {
 	if ((dtmf_integer == OPERATOR_KEY) &&
 		(ast_test_flag(flags, OPTION_OPERATOR_EXIT))) {
-		pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "OPERATOR");
-		return -1;
+		return RESPONSE_OPERATOR;
 	}
 
 	if ((dtmf_integer == terminator) ||
 		(ast_test_flag(flags, OPTION_ANY_TERMINATE))) {
-		pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "DTMF");
+		return RESPONSE_DTMF;
+	}
+
+	return RESPONSE_NO_MATCH;
+}
+
+static int create_destination_directory(const char *path)
+{
+	int res;
+	char directory[PATH_MAX], *file_sep;
+
+	if (!(file_sep = strrchr(path, '/'))) {
+		/* No directory to create */
+		return 0;
+	}
+
+	/* Overwrite temporarily */
+	*file_sep = '\0';
+
+	/* Absolute path? */
+	if (path[0] == '/') {
+		res = ast_mkdir(path, 0777);
+		*file_sep = '/';
+		return res;
+	}
+
+	/* Relative path */
+	res = snprintf(directory, sizeof(directory), "%s/sounds/%s",
+				   ast_config_AST_DATA_DIR, path);
+
+	*file_sep = '/';
+
+	if (res >= sizeof(directory)) {
+		/* We truncated, so we fail */
 		return -1;
 	}
 
-	return 0;
+	return ast_mkdir(directory, 0777);
 }
 
 static int record_exec(struct ast_channel *chan, const char *data)
 {
 	int res = 0;
-	int count = 0;
 	char *ext = NULL, *opts[0];
-	char *parse, *dir, *file;
+	char *parse;
 	int i = 0;
-	char tmp[256];
+	char tmp[PATH_MAX];
 
 	struct ast_filestream *s = NULL;
 	struct ast_frame *f = NULL;
-	
+
 	struct ast_dsp *sildet = NULL;   	/* silence detector dsp */
 	int totalsilence = 0;
 	int dspsilence = 0;
 	int silence = 0;		/* amount of silence to allow */
 	int gotsilence = 0;		/* did we timeout for silence? */
+	int truncate_silence = 1;	/* truncate on complete silence recording */
 	int maxduration = 0;		/* max duration of recording in milliseconds */
 	int gottimeout = 0;		/* did we timeout for maxduration exceeded? */
 	int terminator = '#';
@@ -209,6 +252,7 @@ static int record_exec(struct ast_channel *chan, const char *data)
 	);
 	int ms;
 	struct timeval start;
+	const char *status_response = "ERROR";
 
 	/* The next few lines of code parse out the filename and header from the input string */
 	if (ast_strlen_zero(data)) { /* no data implies no filename or anything is present */
@@ -223,8 +267,6 @@ static int record_exec(struct ast_channel *chan, const char *data)
 		ast_app_parse_options(app_opts, &flags, opts, args.options);
 
 	if (!ast_strlen_zero(args.filename)) {
-		if (strstr(args.filename, "%d"))
-			ast_set_flag(&flags, FLAG_HAS_PERCENT);
 		ext = strrchr(args.filename, '.'); /* to support filename with a . in the filename, not format */
 		if (!ext)
 			ext = strchr(args.filename, ':');
@@ -245,7 +287,10 @@ static int record_exec(struct ast_channel *chan, const char *data)
 			ast_log(LOG_WARNING, "'%s' is not a valid silence duration\n", args.silence);
 		}
 	}
-	
+
+	if (ast_test_flag(&flags, OPTION_NO_TRUNCATE))
+		truncate_silence = 0;
+
 	if (args.maxduration) {
 		if ((sscanf(args.maxduration, "%30d", &i) == 1) && (i > -1))
 			/* Convert duration to milliseconds */
@@ -259,45 +304,38 @@ static int record_exec(struct ast_channel *chan, const char *data)
 	if (ast_test_flag(&flags, OPTION_IGNORE_TERMINATE))
 		terminator = '\0';
 
-	/* done parsing */
+	/*
+	  If a '%d' is specified as part of the filename, we replace that token with
+	  sequentially incrementing numbers until we find a unique filename.
+	*/
+	if (strchr(args.filename, '%')) {
+		size_t src, dst, count = 0;
+		size_t src_len = strlen(args.filename);
+		size_t dst_len = sizeof(tmp) - 1;
 
-	/* these are to allow the use of the %d in the config file for a wild card of sort to
-	  create a new file with the inputed name scheme */
-	if (ast_test_flag(&flags, FLAG_HAS_PERCENT)) {
-		AST_DECLARE_APP_ARGS(fname,
-			AST_APP_ARG(piece)[100];
-		);
-		char *tmp2 = ast_strdupa(args.filename);
-		char countstring[15];
-		int idx;
-
-		/* Separate each piece out by the format specifier */
-		AST_NONSTANDARD_APP_ARGS(fname, tmp2, '%');
 		do {
-			int tmplen;
-			/* First piece has no leading percent, so it's copied verbatim */
-			ast_copy_string(tmp, fname.piece[0], sizeof(tmp));
-			tmplen = strlen(tmp);
-			for (idx = 1; idx < fname.argc; idx++) {
-				if (fname.piece[idx][0] == 'd') {
-					/* Substitute the count */
-					snprintf(countstring, sizeof(countstring), "%d", count);
-					ast_copy_string(tmp + tmplen, countstring, sizeof(tmp) - tmplen);
-					tmplen += strlen(countstring);
-				} else if (tmplen + 2 < sizeof(tmp)) {
-					/* Unknown format specifier - just copy it verbatim */
-					tmp[tmplen++] = '%';
-					tmp[tmplen++] = fname.piece[idx][0];
+			for (src = 0, dst = 0; src < src_len && dst < dst_len; src++) {
+				if (!strncmp(&args.filename[src], "%d", 2)) {
+					int s = snprintf(&tmp[dst], PATH_MAX - dst, "%zu", count);
+					if (s >= PATH_MAX - dst) {
+						/* We truncated, so we need to bail */
+						ast_log(LOG_WARNING, "Failed to create unique filename from template: %s\n", args.filename);
+						pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "ERROR");
+						return -1;
+					}
+					dst += s;
+					src++;
+				} else {
+					tmp[dst] = args.filename[src];
+					tmp[++dst] = '\0';
 				}
-				/* Copy the remaining portion of the piece */
-				ast_copy_string(tmp + tmplen, &(fname.piece[idx][1]), sizeof(tmp) - tmplen);
 			}
 			count++;
 		} while (ast_fileexists(tmp, ext, ast_channel_language(chan)) > 0);
-		pbx_builtin_setvar_helper(chan, "RECORDED_FILE", tmp);
 	} else
 		ast_copy_string(tmp, args.filename, sizeof(tmp));
-	/* end of routine mentioned */
+
+	pbx_builtin_setvar_helper(chan, "RECORDED_FILE", tmp);
 
 	if (ast_channel_state(chan) != AST_STATE_UP) {
 		if (ast_test_flag(&flags, OPTION_SKIP)) {
@@ -312,7 +350,7 @@ static int record_exec(struct ast_channel *chan, const char *data)
 
 	if (res) {
 		ast_log(LOG_WARNING, "Could not answer channel '%s'\n", ast_channel_name(chan));
-		pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "ERROR");
+		status_response = "ERROR";
 		goto out;
 	}
 
@@ -322,7 +360,8 @@ static int record_exec(struct ast_channel *chan, const char *data)
 		if (!res) {
 			res = ast_waitstream(chan, "");
 		} else {
-			ast_log(LOG_WARNING, "ast_streamfile failed on %s\n", ast_channel_name(chan));
+			ast_log(LOG_WARNING, "ast_streamfile(beep) failed on %s\n", ast_channel_name(chan));
+			res = 0;
 		}
 		ast_stopstream(chan);
 	}
@@ -344,20 +383,20 @@ static int record_exec(struct ast_channel *chan, const char *data)
 			return -1;
 		}
 		ast_dsp_set_threshold(sildet, ast_dsp_get_threshold_from_settings(THRESHOLD_SILENCE));
-	} 
+	}
 
-	/* Create the directory if it does not exist. */
-	dir = ast_strdupa(tmp);
-	if ((file = strrchr(dir, '/')))
-		*file++ = '\0';
-	ast_mkdir (dir, 0777);
+	if (create_destination_directory(tmp)) {
+		ast_log(LOG_WARNING, "Could not create directory for file %s\n", args.filename);
+		status_response = "ERROR";
+		goto out;
+	}
 
 	ioflags = ast_test_flag(&flags, OPTION_APPEND) ? O_CREAT|O_APPEND|O_WRONLY : O_CREAT|O_TRUNC|O_WRONLY;
 	s = ast_writefile(tmp, ext, NULL, ioflags, 0, AST_FILE_MODE);
 
 	if (!s) {
 		ast_log(LOG_WARNING, "Could not create file %s\n", args.filename);
-		pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "ERROR");
+		status_response = "ERROR";
 		goto out;
 	}
 
@@ -392,7 +431,7 @@ static int record_exec(struct ast_channel *chan, const char *data)
 			if (res) {
 				ast_log(LOG_WARNING, "Problem writing frame\n");
 				ast_frfree(f);
-				pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "ERROR");
+				status_response = "ERROR";
 				break;
 			}
 
@@ -408,7 +447,7 @@ static int record_exec(struct ast_channel *chan, const char *data)
 					/* Ended happily with silence */
 					ast_frfree(f);
 					gotsilence = 1;
-					pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "SILENCE");
+					status_response = "SILENCE";
 					break;
 				}
 			}
@@ -417,12 +456,26 @@ static int record_exec(struct ast_channel *chan, const char *data)
 
 			if (res) {
 				ast_log(LOG_WARNING, "Problem writing frame\n");
-				pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "ERROR");
+				status_response = "ERROR";
 				ast_frfree(f);
 				break;
 			}
 		} else if (f->frametype == AST_FRAME_DTMF) {
-			if (record_dtmf_response(chan, &flags, f->subclass.integer, terminator)) {
+			enum dtmf_response rc =
+				record_dtmf_response(chan, &flags, f->subclass.integer, terminator);
+			switch(rc) 	{
+			case RESPONSE_NO_MATCH:
+				break;
+			case RESPONSE_OPERATOR:
+				status_response = "OPERATOR";
+				ast_debug(1, "Got OPERATOR\n");
+				break;
+			case RESPONSE_DTMF:
+				status_response = "DTMF";
+				ast_debug(1, "Got DTMF\n");
+				break;
+			}
+			if (rc != RESPONSE_NO_MATCH) {
 				ast_frfree(f);
 				break;
 			}
@@ -432,19 +485,19 @@ static int record_exec(struct ast_channel *chan, const char *data)
 
 	if (maxduration > 0 && !ms) {
 		gottimeout = 1;
-		pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "TIMEOUT");
+		status_response = "TIMEOUT";
 	}
 
 	if (!f) {
 		ast_debug(1, "Got hangup\n");
 		res = -1;
-		pbx_builtin_setvar_helper(chan, "RECORD_STATUS", "HANGUP");
+		status_response = "HANGUP";
 		if (!ast_test_flag(&flags, OPTION_KEEP)) {
 			ast_filedelete(args.filename, NULL);
 		}
 	}
 
-	if (gotsilence) {
+	if (gotsilence && truncate_silence) {
 		ast_stream_rewind(s, silence - 1000);
 		ast_truncstream(s);
 	} else if (!gottimeout && f) {
@@ -473,6 +526,9 @@ out:
 	if (sildet) {
 		ast_dsp_free(sildet);
 	}
+
+	pbx_builtin_setvar_helper(chan, "RECORD_STATUS", status_response);
+
 	return res;
 }
 
