@@ -21,7 +21,7 @@
  * \brief Stack applications Gosub, Return, etc.
  *
  * \author Tilghman Lesher <app_stack_v003@the-tilghman.com>
- * 
+ *
  * \ingroup applications
  */
 
@@ -31,8 +31,8 @@
  ***/
 
 #include "asterisk.h"
- 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 422719 $")
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
@@ -305,7 +305,7 @@ static void gosub_release_frame(struct ast_channel *chan, struct gosub_stack_fra
 	 */
 	while ((vardata = AST_LIST_REMOVE_HEAD(&frame->varshead, entries))) {
 		if (chan)
-			pbx_builtin_setvar_helper(chan, ast_var_name(vardata), NULL);	
+			pbx_builtin_setvar_helper(chan, ast_var_name(vardata), NULL);
 		ast_var_delete(vardata);
 	}
 
@@ -315,13 +315,14 @@ static void gosub_release_frame(struct ast_channel *chan, struct gosub_stack_fra
 static struct gosub_stack_frame *gosub_allocate_frame(const char *context, const char *extension, int priority, int in_subroutine, unsigned char arguments)
 {
 	struct gosub_stack_frame *new = NULL;
-	int len_extension = strlen(extension), len_context = strlen(context);
+	int len_extension = strlen(extension) + 1;
+	int len_context = strlen(context) + 1;
 
-	if ((new = ast_calloc(1, sizeof(*new) + 2 + len_extension + len_context))) {
+	if ((new = ast_calloc(1, sizeof(*new) + len_extension + len_context))) {
 		AST_LIST_HEAD_INIT_NOLOCK(&new->varshead);
-		strcpy(new->extension, extension);
-		new->context = new->extension + len_extension + 1;
-		strcpy(new->context, context);
+		ast_copy_string(new->extension, extension, len_extension);
+		new->context = new->extension + len_extension;
+		ast_copy_string(new->context, context, len_context);
 		new->priority = priority;
 		new->in_subroutine = in_subroutine ? 1 : 0;
 		new->arguments = arguments;
@@ -1322,5 +1323,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT | AST_MODFLAG_LOAD_ORDER,
 		.load = load_module,
 		.unload = unload_module,
 		.load_pri = AST_MODPRI_APP_DEPEND,
-		.nonoptreq = "res_agi",
 		);
