@@ -26,6 +26,7 @@
  */
 
 /*** MODULEINFO
+	<use type="module">res_monitor</use>
 	<support_level>core</support_level>
  ***/
 
@@ -202,6 +203,8 @@ static void start_automonitor(struct ast_bridge_channel *bridge_channel, struct
 		return;
 	}
 
+	ast_monitor_setjoinfiles(peer_chan, 1);
+
 	if (features_cfg && !ast_strlen_zero(features_cfg->courtesytone)) {
 		ast_bridge_channel_queue_playfile(bridge_channel, NULL, features_cfg->courtesytone, NULL);
 		ast_bridge_channel_write_playfile(bridge_channel, NULL, features_cfg->courtesytone, NULL);
@@ -212,6 +215,7 @@ static void start_automonitor(struct ast_bridge_channel *bridge_channel, struct
 		ast_bridge_channel_write_playfile(bridge_channel, NULL, start_message, NULL);
 	}
 
+	pbx_builtin_setvar_helper(bridge_channel->chan, "TOUCH_MONITOR_OUTPUT", touch_filename);
 	pbx_builtin_setvar_helper(peer_chan, "TOUCH_MONITOR_OUTPUT", touch_filename);
 }
 
@@ -399,6 +403,7 @@ static void start_automixmonitor(struct ast_bridge_channel *bridge_channel, stru
 		ast_bridge_channel_write_playfile(bridge_channel, NULL, start_message, NULL);
 	}
 
+	pbx_builtin_setvar_helper(bridge_channel->chan, "TOUCH_MIXMONITOR_OUTPUT", touch_filename);
 	pbx_builtin_setvar_helper(peer_chan, "TOUCH_MIXMONITOR_OUTPUT", touch_filename);
 }
 
