@@ -35,11 +35,7 @@
 
 ASTERISK_FILE_VERSION(__FILE__, "$Rev$")
 
-#ifdef FREERADIUS_CLIENT
-#include <freeradius-client.h>
-#else
-#include <radiusclient-ng.h>
-#endif
+#include RADIUS_HEADER_STR
 
 #include "asterisk/channel.h"
 #include "asterisk/cel.h"
@@ -95,7 +91,7 @@ static rc_handle *rh = NULL;
 
 #define RADIUS_BACKEND_NAME "CEL Radius Logging"
 
-#define ADD_VENDOR_CODE(x,y) (rc_avpair_add(rh, send, x, &y, strlen(y), VENDOR_CODE))
+#define ADD_VENDOR_CODE(x,y) (rc_avpair_add(rh, send, x, (void *)y, strlen(y), VENDOR_CODE))
 
 static int build_radius_record(VALUE_PAIR **send, struct ast_cel_event_record *record)
 {
@@ -176,7 +172,7 @@ static int build_radius_record(VALUE_PAIR **send, struct ast_cel_event_record *r
 	/* Setting Acct-Session-Id & User-Name attributes for proper generation
 	   of Acct-Unique-Session-Id on server side */
 	/* Channel */
-	if (!rc_avpair_add(rh, send, PW_USER_NAME, &record->channel_name,
+	if (!rc_avpair_add(rh, send, PW_USER_NAME, (void *)record->channel_name,
 			strlen(record->channel_name), 0)) {
 		return -1;
 	}
