@@ -110,7 +110,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #endif
 /**/
 
-static const char desc[] = "VoiceTronix V6PCI/V12PCI/V4PCI  API Support";
 static const char tdesc[] = "Standard VoiceTronix API Driver";
 static const char config[] = "vpb.conf";
 
@@ -362,63 +361,63 @@ static int vpb_indicate(struct ast_channel *ast, int condition, const void *data
 static int vpb_fixup(struct ast_channel *oldchan, struct ast_channel *newchan);
 
 static struct ast_channel_tech vpb_tech = {
-	type: "vpb",
-	description: tdesc,
-	capabilities: NULL,
-	properties: 0,
-	requester: vpb_request,
-	devicestate: NULL,
-	send_digit_begin: vpb_digit_begin,
-	send_digit_end: vpb_digit_end,
-	call: vpb_call,
-	hangup: vpb_hangup,
-	answer: vpb_answer,
-	read: vpb_read,
-	write: vpb_write,
-	send_text: NULL,
-	send_image: NULL,
-	send_html: NULL,
-	exception: NULL,
-	early_bridge: NULL,
-	indicate: vpb_indicate,
-	fixup: vpb_fixup,
-	setoption: NULL,
-	queryoption: NULL,
-	transfer: NULL,
-	write_video: NULL,
-	write_text: NULL,
-	func_channel_read: NULL,
-	func_channel_write: NULL,
+	.type = "vpb",
+	.description = tdesc,
+	.capabilities = NULL,
+	.properties = 0,
+	.requester = vpb_request,
+	.devicestate = NULL,
+	.send_digit_begin = vpb_digit_begin,
+	.send_digit_end = vpb_digit_end,
+	.call = vpb_call,
+	.hangup = vpb_hangup,
+	.answer = vpb_answer,
+	.read = vpb_read,
+	.write = vpb_write,
+	.send_text = NULL,
+	.send_image = NULL,
+	.send_html = NULL,
+	.exception = NULL,
+	.early_bridge = NULL,
+	.indicate = vpb_indicate,
+	.fixup = vpb_fixup,
+	.setoption = NULL,
+	.queryoption = NULL,
+	.transfer = NULL,
+	.write_video = NULL,
+	.write_text = NULL,
+	.func_channel_read = NULL,
+	.func_channel_write = NULL,
 };
 
 static struct ast_channel_tech vpb_tech_indicate = {
-	type: "vpb",
-	description: tdesc,
-	capabilities: NULL,
-	properties: 0,
-	requester: vpb_request,
-	devicestate: NULL,
-	send_digit_begin: vpb_digit_begin,
-	send_digit_end: vpb_digit_end,
-	call: vpb_call,
-	hangup: vpb_hangup,
-	answer: vpb_answer,
-	read: vpb_read,
-	write: vpb_write,
-	send_text: NULL,
-	send_image: NULL,
-	send_html: NULL,
-	exception: NULL,
-	early_bridge: NULL,
-	indicate: NULL,
-	fixup: vpb_fixup,
-	setoption: NULL,
-	queryoption: NULL,
-	transfer: NULL,
-	write_video: NULL,
-	write_text: NULL,
-	func_channel_read: NULL,
-	func_channel_write: NULL,
+	.type = "vpb",
+	.description = tdesc,
+	.capabilities = NULL,
+	.properties = 0,
+	.requester = vpb_request,
+	.devicestate = NULL,
+	.send_digit_begin = vpb_digit_begin,
+	.send_digit_end = vpb_digit_end,
+	.call = vpb_call,
+	.hangup = vpb_hangup,
+	.answer = vpb_answer,
+	.read = vpb_read,
+	.write = vpb_write,
+	.send_text = NULL,
+	.send_image = NULL,
+	.send_html = NULL,
+	.exception = NULL,
+	.early_bridge = NULL,
+	.indicate = NULL,
+	.fixup = vpb_fixup,
+	.setoption = NULL,
+	.queryoption = NULL,
+	.transfer = NULL,
+	.write_video = NULL,
+	.write_text = NULL,
+	.func_channel_read = NULL,
+	.func_channel_write = NULL,
 };
 
 #if defined(VPB_NATIVE_BRIDGING)
@@ -2686,7 +2685,7 @@ static enum ast_module_load_result load_module()
 	ast_format_cap_append(vpb_tech_indicate.capabilities, ast_format_slin, 0);
 	try {
 		num_cards = vpb_get_num_cards();
-	} catch (std::exception e) {
+	} catch (std::exception&) {
 		ast_log(LOG_ERROR, "No Voicetronix cards detected\n");
 		return AST_MODULE_LOAD_DECLINE;
 	}
