@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2011, Digium, Inc.
  *
- * Joshua Colp <jcolp@digium.com> 
+ * Joshua Colp <jcolp@digium.com>
  *
  * See http://www.asterisk.org for more information about
  * the Asterisk project. Please do not directly contact
@@ -20,7 +20,7 @@
  *
  * \brief Technology independent volume control
  *
- * \author Joshua Colp <jcolp@digium.com> 
+ * \author Joshua Colp <jcolp@digium.com>
  *
  * \ingroup functions
  *
@@ -32,7 +32,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 411328 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
@@ -72,8 +72,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 411328 $")
 
 struct volume_information {
 	struct ast_audiohook audiohook;
-	int tx_gain;
-	int rx_gain;
+	float tx_gain;
+	float rx_gain;
 	unsigned int flags;
 };
 
@@ -109,7 +109,7 @@ static int volume_callback(struct ast_audiohook *audiohook, struct ast_channel *
 {
 	struct ast_datastore *datastore = NULL;
 	struct volume_information *vi = NULL;
-	int *gain = NULL;
+	float *gain = NULL;
 
 	/* If the audiohook is stopping it means the channel is shutting down.... but we let the datastore destroy take care of it */
 	if (audiohook->status == AST_AUDIOHOOK_STATUS_DONE)
@@ -126,7 +126,7 @@ static int volume_callback(struct ast_audiohook *audiohook, struct ast_channel *
 		if (frame->frametype == AST_FRAME_DTMF) {
 			/* Only use DTMF coming from the source... not going to it */
 			if (direction != AST_AUDIOHOOK_DIRECTION_READ)
-				return 0; 
+				return 0;
 			if (frame->subclass.integer == '*') {
 				vi->tx_gain += 1;
 				vi->rx_gain += 1;
@@ -137,13 +137,13 @@ static int volume_callback(struct ast_audiohook *audiohook, struct ast_channel *
 		}
 	}
 
-	
+
 	if (frame->frametype == AST_FRAME_VOICE) {
 		/* Based on direction of frame grab the gain, and confirm it is applicable */
 		if (!(gain = (direction == AST_AUDIOHOOK_DIRECTION_READ) ? &vi->rx_gain : &vi->tx_gain) || !*gain)
 			return 0;
 		/* Apply gain to frame... easy as pi */
-		ast_frame_adjust_volume(frame, *gain);
+		ast_frame_adjust_volume_float(frame, *gain);
 	}
 
 	return 0;
@@ -194,10 +194,10 @@ static int volume_write(struct ast_channel *chan, const char *cmd, char *data, c
 		return -1;
 	}
 
-	if (!strcasecmp(args.direction, "tx")) { 
-		vi->tx_gain = atoi(value); 
+	if (!strcasecmp(args.direction, "tx")) {
+		vi->tx_gain = atof(value);
 	} else if (!strcasecmp(args.direction, "rx")) {
-		vi->rx_gain = atoi(value);
+		vi->rx_gain = atof(value);
 	} else {
 		ast_log(LOG_ERROR, "Direction must be either RX or TX\n");
 	}
@@ -211,13 +211,13 @@ static int volume_write(struct ast_channel *chan, const char *cmd, char *data, c
 	}
 
 	/* Add Option data to struct */
-	
+
 	if (!ast_strlen_zero(args.options)) {
 		struct ast_flags flags = { 0 };
 		ast_app_parse_options(volume_opts, &flags, NULL, args.options);
 		vi->flags = flags.flags;
-	} else { 
-		vi->flags = 0; 
+	} else {
+		vi->flags = 0;
 	}
 
 	return 0;
