@@ -53,7 +53,7 @@ static struct fdleaks {
 	unsigned int isopen:1;
 	char file[40];
 	char function[25];
-	char callargs[60];
+	char callargs[100];
 } fdleaks[1024] = { { "", }, };
 
 /* COPY does ast_copy_string(dst, src, sizeof(dst)), except:
@@ -271,7 +271,7 @@ static char *handle_show_fd(struct ast_cli_entry *e, int cmd, struct ast_cli_arg
 	case CLI_GENERATE:
 		return NULL;
 	}
-	getrlimit(RLIMIT_FSIZE, &rl);
+	getrlimit(RLIMIT_NOFILE, &rl);
 	if (rl.rlim_cur == RLIM_INFINITY || rl.rlim_max == RLIM_INFINITY) {
 		ast_copy_string(line, "unlimited", sizeof(line));
 	} else {
@@ -306,4 +306,3 @@ int ast_fd_init(void)
 	return 0;
 }
 #endif /* defined(DEBUG_FD_LEAKS) */
-
