@@ -185,21 +185,29 @@ void ast_free_ptr(void *ptr)
 	ast_free(ptr);
 }
 
-static void print_backtrace(struct ast_bt *bt)
+static void print_backtrace(struct ast_bt *bt, struct ast_cli_args *a)
 {
 	int i = 0;
-	char **strings;
+	struct ast_vector_string *strings;
 
 	if (!bt) {
 		return;
 	}
 
 	if ((strings = ast_bt_get_symbols(bt->addresses, bt->num_frames))) {
-		astmm_log("Memory allocation backtrace:\n");
-		for (i = 3; i < bt->num_frames - 2; i++) {
-			astmm_log("#%d: [%p] %s\n", i - 3, bt->addresses[i], strings[i]);
+		if (a) {
+			ast_cli(a->fd, "Memory allocation backtrace:\n");
+		} else {
+			astmm_log("Memory allocation backtrace:\n");
 		}
-		ast_std_free(strings);
+		for (i = 3; i < AST_VECTOR_SIZE(strings) - 2; i++) {
+			if (a) {
+				ast_cli(a->fd, "#%d: %s\n", i - 3, AST_VECTOR_GET(strings, i));
+			} else {
+				astmm_log("#%d: %s\n", i - 3, AST_VECTOR_GET(strings, i));
+			}
+		}
+		ast_bt_free_symbols(strings);
 	}
 }
 
@@ -227,6 +235,8 @@ static void *__ast_alloc_region(size_t size, const enum func_type which, const c
 	unsigned int *fence;
 	int hash;
 
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
+
 	if (!(reg = malloc(size + sizeof(*reg) + sizeof(*fence)))) {
 		astmm_log("Memory Allocation Failure - '%d' bytes at %s %s() line %d\n",
 			(int) size, file, func, lineno);
@@ -309,7 +319,7 @@ static void region_data_check(struct ast_region *reg)
 		if (*pos != FREED_MAGIC) {
 			astmm_log("WARNING: Memory corrupted after free of %p allocated at %s %s() line %d\n",
 				reg->data, reg->file, reg->func, reg->lineno);
-			print_backtrace(reg->bt);
+			print_backtrace(reg->bt, NULL);
 			my_do_crash();
 			break;
 		}
@@ -429,14 +439,14 @@ static void region_check_fences(struct ast_region *reg)
 	if (*fence != FENCE_MAGIC) {
 		astmm_log("WARNING: Low fence violation of %p allocated at %s %s() line %d\n",
 			reg->data, reg->file, reg->func, reg->lineno);
-		print_backtrace(reg->bt);
+		print_backtrace(reg->bt, NULL);
 		my_do_crash();
 	}
 	fence = (unsigned int *) (reg->data + reg->len);
 	if (get_unaligned_uint32(fence) != FENCE_MAGIC) {
 		astmm_log("WARNING: High fence violation of %p allocated at %s %s() line %d\n",
 			reg->data, reg->file, reg->func, reg->lineno);
-		print_backtrace(reg->bt);
+		print_backtrace(reg->bt, NULL);
 		my_do_crash();
 	}
 }
@@ -664,7 +674,6 @@ int __ast_vasprintf(char **strp, const char *fmt, va_list ap, const char *file,
 	size = vsnprintf(&s, 1, fmt, ap2);
 	va_end(ap2);
 	if (!(*strp = __ast_alloc_region(size + 1, FUNC_VASPRINTF, file, lineno, func, 0))) {
-		va_end(ap);
 		return -1;
 	}
 	vsnprintf(*strp, size + 1, fmt, ap);
@@ -881,6 +890,9 @@ static char *handle_memory_show_allocations(struct ast_cli_entry *e, int cmd, st
 			ast_cli(a->fd, "%10u bytes allocated%s by %20s() line %5u of %s\n",
 				(unsigned int) reg->len, reg->cache ? " (cache)" : "",
 				reg->func, reg->lineno, reg->file);
+			if (reg->bt && !ast_strlen_zero(fn)) {
+				print_backtrace(reg->bt, a);
+			}
 
 			selected_len += reg->len;
 			if (reg->cache) {
