@@ -292,7 +292,6 @@ int ast_dial_append(struct ast_dial *dial, const char *tech, const char *device,
 /*! \brief Helper function that requests all channels */
 static int begin_dial_prerun(struct ast_dial_channel *channel, struct ast_channel *chan, struct ast_format_cap *cap, const char *predial_string)
 {
-	char numsubst[AST_MAX_EXTENSION];
 	struct ast_format_cap *cap_all_audio = NULL;
 	struct ast_format_cap *cap_request;
 	struct ast_format_cap *requester_cap = NULL;
@@ -315,9 +314,6 @@ static int begin_dial_prerun(struct ast_dial_channel *channel, struct ast_channe
 		}
 	}
 
-	/* Copy device string over */
-	ast_copy_string(numsubst, channel->device, sizeof(numsubst));
-
 	if (cap && ast_format_cap_count(cap)) {
 		cap_request = cap;
 	} else if (requester_cap) {
@@ -329,7 +325,7 @@ static int begin_dial_prerun(struct ast_dial_channel *channel, struct ast_channe
 	}
 
 	/* If we fail to create our owner channel bail out */
-	if (!(channel->owner = ast_request(channel->tech, cap_request, &assignedids, chan, numsubst, &channel->cause))) {
+	if (!(channel->owner = ast_request(channel->tech, cap_request, &assignedids, chan, channel->device, &channel->cause))) {
 		ao2_cleanup(cap_all_audio);
 		return -1;
 	}
@@ -411,21 +407,25 @@ int ast_dial_prerun(struct ast_dial *dial, struct ast_channel *chan, struct ast_
 }
 
 /*! \brief Helper function that does the beginning dialing per-appended channel */
-static int begin_dial_channel(struct ast_dial_channel *channel, struct ast_channel *chan, int async, const char *predial_string)
+static int begin_dial_channel(struct ast_dial_channel *channel, struct ast_channel *chan, int async, const char *predial_string, struct ast_channel *forwarder_chan)
 {
-	char numsubst[AST_MAX_EXTENSION];
 	int res = 1;
+	char forwarder[AST_CHANNEL_NAME];
 
 	/* If no owner channel exists yet execute pre-run */
 	if (!channel->owner && begin_dial_prerun(channel, chan, NULL, predial_string)) {
 		return 0;
 	}
 
-	/* Copy device string over */
-	ast_copy_string(numsubst, channel->device, sizeof(numsubst));
+	if (forwarder_chan) {
+		ast_copy_string(forwarder, ast_channel_name(forwarder_chan), sizeof(forwarder));
+		ast_channel_lock(channel->owner);
+		pbx_builtin_setvar_helper(channel->owner, "FORWARDERNAME", forwarder);
+		ast_channel_unlock(channel->owner);
+	}
 
 	/* Attempt to actually call this device */
-	if ((res = ast_call(channel->owner, numsubst, 0))) {
+	if ((res = ast_call(channel->owner, channel->device, 0))) {
 		res = 0;
 		ast_hangup(channel->owner);
 		channel->owner = NULL;
@@ -435,7 +435,7 @@ static int begin_dial_channel(struct ast_dial_channel *channel, struct ast_chann
 		}
 		ast_channel_publish_dial(async ? NULL : chan, channel->owner, channel->device, NULL);
 		res = 1;
-		ast_verb(3, "Called %s\n", numsubst);
+		ast_verb(3, "Called %s\n", channel->device);
 	}
 
 	return res;
@@ -451,7 +451,7 @@ static int begin_dial(struct ast_dial *dial, struct ast_channel *chan, int async
 	/* Iterate through channel list, requesting and calling each one */
 	AST_LIST_LOCK(&dial->channels);
 	AST_LIST_TRAVERSE(&dial->channels, channel, list) {
-		success += begin_dial_channel(channel, chan, async, predial_string);
+		success += begin_dial_channel(channel, chan, async, predial_string, NULL);
 	}
 	AST_LIST_UNLOCK(&dial->channels);
 
@@ -507,7 +507,7 @@ static int handle_call_forward(struct ast_dial *dial, struct ast_dial_channel *c
 	channel->owner = NULL;
 
 	/* Finally give it a go... send it out into the world */
-	begin_dial_channel(channel, chan, chan ? 0 : 1, predial_string);
+	begin_dial_channel(channel, chan, chan ? 0 : 1, predial_string, original);
 
 	ast_channel_publish_dial_forward(chan, original, channel->owner, NULL, "CANCEL",
 		ast_channel_call_forward(original));
