@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 421678 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/logger.h"
 #include "asterisk/format.h"
@@ -192,6 +192,11 @@ struct ast_format *ast_format_mp4;
  */
 struct ast_format *ast_format_vp8;
 
+/*!
+ * \brief Built-in cached vp9 format.
+ */
+struct ast_format *ast_format_vp9;
+
 /*!
  * \brief Built-in cached jpeg format.
  */
@@ -232,6 +237,14 @@ struct ast_format *ast_format_t140_red;
  */
 struct ast_format *ast_format_none;
 
+/*!
+ * \brief Built-in "silk" format
+ */
+struct ast_format *ast_format_silk8;
+struct ast_format *ast_format_silk12;
+struct ast_format *ast_format_silk16;
+struct ast_format *ast_format_silk24;
+
 /*! \brief Number of buckets to use for the media format cache (should be prime for performance reasons) */
 #define CACHE_BUCKETS 53
 
@@ -328,20 +341,25 @@ static void format_cache_shutdown(void)
 	ao2_replace(ast_format_h264, NULL);
 	ao2_replace(ast_format_mp4, NULL);
 	ao2_replace(ast_format_vp8, NULL);
+	ao2_replace(ast_format_vp9, NULL);
 	ao2_replace(ast_format_t140_red, NULL);
 	ao2_replace(ast_format_t140, NULL);
 	ao2_replace(ast_format_none, NULL);
+	ao2_replace(ast_format_silk8, NULL);
+	ao2_replace(ast_format_silk12, NULL);
+	ao2_replace(ast_format_silk16, NULL);
+	ao2_replace(ast_format_silk24, NULL);
 }
 
 int ast_format_cache_init(void)
 {
-	formats = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_RWLOCK, CACHE_BUCKETS,
-		format_hash_cb, format_cmp_cb);
+	formats = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, CACHE_BUCKETS,
+		format_hash_cb, NULL, format_cmp_cb);
 	if (!formats) {
 		return -1;
 	}
 
-	ast_register_atexit(format_cache_shutdown);
+	ast_register_cleanup(format_cache_shutdown);
 
 	return 0;
 }
@@ -420,12 +438,22 @@ static void set_cached_format(const char *name, struct ast_format *format)
 		ao2_replace(ast_format_mp4, format);
 	} else if (!strcmp(name, "vp8")) {
 		ao2_replace(ast_format_vp8, format);
+	} else if (!strcmp(name, "vp9")) {
+		ao2_replace(ast_format_vp9, format);
 	} else if (!strcmp(name, "red")) {
 		ao2_replace(ast_format_t140_red, format);
 	} else if (!strcmp(name, "t140")) {
 		ao2_replace(ast_format_t140, format);
 	} else if (!strcmp(name, "none")) {
 		ao2_replace(ast_format_none, format);
+	} else if (!strcmp(name, "silk8")) {
+		ao2_replace(ast_format_silk8, format);
+	} else if (!strcmp(name, "silk12")) {
+		ao2_replace(ast_format_silk12, format);
+	} else if (!strcmp(name, "silk16")) {
+		ao2_replace(ast_format_silk16, format);
+	} else if (!strcmp(name, "silk24")) {
+		ao2_replace(ast_format_silk24, format);
 	}
 }
 
@@ -500,6 +528,7 @@ struct ast_format *ast_format_cache_get_slin_by_rate(unsigned int rate)
 int ast_format_cache_is_slinear(struct ast_format *format)
 {
 	if ((ast_format_cmp(format, ast_format_slin) == AST_FORMAT_CMP_EQUAL)
+		|| (ast_format_cmp(format, ast_format_slin12) == AST_FORMAT_CMP_EQUAL)
 		|| (ast_format_cmp(format, ast_format_slin16) == AST_FORMAT_CMP_EQUAL)
 		|| (ast_format_cmp(format, ast_format_slin24) == AST_FORMAT_CMP_EQUAL)
 		|| (ast_format_cmp(format, ast_format_slin32) == AST_FORMAT_CMP_EQUAL)
@@ -513,3 +542,23 @@ int ast_format_cache_is_slinear(struct ast_format *format)
 	return 0;
 }
 
+struct ast_format *ast_format_cache_get_by_codec(const struct ast_codec *codec)
+{
+	struct ast_format *format;
+	struct ao2_iterator it;
+
+	for (it = ao2_iterator_init(formats, 0);
+		 (format = ao2_iterator_next(&it));
+		 ao2_ref(format, -1)) {
+		struct ast_codec *candidate = ast_format_get_codec(format);
+		if (codec == candidate) {
+			ao2_cleanup(candidate);
+			ao2_iterator_destroy(&it);
+			return format;
+		}
+		ao2_cleanup(candidate);
+	}
+
+	ao2_iterator_destroy(&it);
+	return NULL;
+}
