@@ -44,6 +44,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			<syntax>
 				<channel_snapshot/>
 			</syntax>
+			<see-also>
+				<ref type="managerEvent">Newstate</ref>
+				<ref type="managerEvent">Hangup</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="Newstate">
@@ -52,6 +56,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			<syntax>
 				<channel_snapshot/>
 			</syntax>
+			<see-also>
+				<ref type="managerEvent">Newchannel</ref>
+				<ref type="managerEvent">Hangup</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="Hangup">
@@ -66,6 +74,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<para>A description of why the channel was hung up.</para>
 				</parameter>
 			</syntax>
+			<see-also>
+				<ref type="managerEvent">Newchannel</ref>
+				<ref type="managerEvent">SoftHangupRequest</ref>
+				<ref type="managerEvent">HangupRequest</ref>
+				<ref type="managerEvent">Newstate</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="HangupRequest">
@@ -75,6 +89,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 				<channel_snapshot/>
 				<xi:include xpointer="xpointer(/docs/managerEvent[@name='Hangup']/managerEventInstance/syntax/parameter[@name='Cause'])" />
 			</syntax>
+			<see-also>
+				<ref type="managerEvent">SoftHangupRequest</ref>
+				<ref type="managerEvent">Hangup</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="SoftHangupRequest">
@@ -84,6 +102,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 				<channel_snapshot/>
 				<xi:include xpointer="xpointer(/docs/managerEvent[@name='Hangup']/managerEventInstance/syntax/parameter[@name='Cause'])" />
 			</syntax>
+			<see-also>
+				<ref type="managerEvent">HangupRequest</ref>
+				<ref type="managerEvent">Hangup</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="NewExten">
@@ -114,6 +136,20 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<para>A description of the Caller ID presentation.</para>
 				</parameter>
 			</syntax>
+			<see-also>
+				<ref type="function">CALLERID</ref>
+			</see-also>
+		</managerEventInstance>
+	</managerEvent>
+	<managerEvent language="en_US" name="NewConnectedLine">
+		<managerEventInstance class="EVENT_FLAG_CALL">
+			<synopsis>Raised when a channel's connected line information is changed.</synopsis>
+			<syntax>
+				<channel_snapshot/>
+			</syntax>
+			<see-also>
+				<ref type="function">CONNECTEDLINE</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="NewAccountCode">
@@ -125,6 +161,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<para>The channel's previous account code</para>
 				</parameter>
 			</syntax>
+			<see-also>
+				<ref type="function">CHANNEL</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="DialBegin">
@@ -139,6 +178,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			</syntax>
 			<see-also>
 				<ref type="application">Dial</ref>
+				<ref type="application">Originate</ref>
+				<ref type="manager">Originate</ref>
+				<ref type="managerEvent">DialEnd</ref>
 			</see-also>
 		</managerEventInstance>
 	</managerEvent>
@@ -192,6 +234,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			</syntax>
 			<see-also>
 				<ref type="application">Dial</ref>
+				<ref type="application">Originate</ref>
+				<ref type="manager">Originate</ref>
+				<ref type="managerEvent">DialBegin</ref>
 			</see-also>
 		</managerEventInstance>
 	</managerEvent>
@@ -204,6 +249,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<para>The suggested MusicClass, if provided.</para>
 				</parameter>
 			</syntax>
+			<see-also>
+				<ref type="managerEvent">Unhold</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="Unhold">
@@ -212,6 +260,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			<syntax>
 				<channel_snapshot/>
 			</syntax>
+			<see-also>
+				<ref type="managerEvent">Hold</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="ChanSpyStart">
@@ -222,7 +273,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 				<channel_snapshot prefix="Spyee"/>
 			</syntax>
 			<see-also>
-				<ref type="application">ChanSpyStop</ref>
+				<ref type="managerEvent">ChanSpyStop</ref>
+				<ref type="application">ChanSpy</ref>
 			</see-also>
 		</managerEventInstance>
 	</managerEvent>
@@ -234,7 +286,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 				<channel_snapshot prefix="Spyee"/>
 			</syntax>
 			<see-also>
-				<ref type="application">ChanSpyStart</ref>
+				<ref type="managerEvent">ChanSpyStart</ref>
+				<ref type="application">ChanSpy</ref>
 			</see-also>
 		</managerEventInstance>
 	</managerEvent>
@@ -247,6 +300,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 					<para>Hangup handler parameter string passed to the Gosub application.</para>
 				</parameter>
 			</syntax>
+			<see-also>
+				<ref type="function">CHANNEL</ref>
+			</see-also>
 		</managerEventInstance>
 	</managerEvent>
 	<managerEvent language="en_US" name="HangupHandlerPop">
@@ -353,6 +409,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 			</syntax>
 			<see-also>
 				<ref type="managerEvent">MusicOnHoldStop</ref>
+				<ref type="application">StartMusicOnHold</ref>
 				<ref type="application">MusicOnHold</ref>
 			</see-also>
 		</managerEventInstance>
@@ -406,16 +463,17 @@ struct ast_str *ast_manager_build_channel_state_string_prefix(
 		const struct ast_channel_snapshot *snapshot,
 		const char *prefix)
 {
-	struct ast_str *out = ast_str_create(1024);
-	int res = 0;
-	char *caller_name, *connected_name;
+	struct ast_str *out;
+	char *caller_name;
+	char *connected_name;
+	int res;
 
-	if (!out) {
+	if (snapshot->tech_properties & AST_CHAN_TP_INTERNAL) {
 		return NULL;
 	}
 
-	if (snapshot->tech_properties & AST_CHAN_TP_INTERNAL) {
-		ast_free(out);
+	out = ast_str_create(1024);
+	if (!out) {
 		return NULL;
 	}
 
@@ -452,10 +510,11 @@ struct ast_str *ast_manager_build_channel_state_string_prefix(
 		prefix, snapshot->uniqueid,
 		prefix, snapshot->linkedid);
 
+	ast_free(caller_name);
+	ast_free(connected_name);
+
 	if (!res) {
 		ast_free(out);
-		ast_free(caller_name);
-		ast_free(connected_name);
 		return NULL;
 	}
 
@@ -471,9 +530,6 @@ struct ast_str *ast_manager_build_channel_state_string_prefix(
 		}
 	}
 
-	ast_free(caller_name);
-	ast_free(connected_name);
-
 	return out;
 }
 
@@ -697,28 +753,33 @@ static void channel_hangup_request_cb(void *data,
 	struct stasis_message *message)
 {
 	struct ast_channel_blob *obj = stasis_message_data(message);
-	RAII_VAR(struct ast_str *, extra, NULL, ast_free);
-	RAII_VAR(struct ast_str *, channel_event_string, NULL, ast_free);
+	struct ast_str *extra;
+	struct ast_str *channel_event_string;
 	struct ast_json *cause;
 	int is_soft;
 	char *manager_event = "HangupRequest";
 
+	if (!obj->snapshot) {
+		/* No snapshot?  Likely an earlier allocation failure creating it. */
+		return;
+	}
+
 	extra = ast_str_create(20);
 	if (!extra) {
 		return;
 	}
 
 	channel_event_string = ast_manager_build_channel_state_string(obj->snapshot);
-
 	if (!channel_event_string) {
+		ast_free(extra);
 		return;
 	}
 
 	cause = ast_json_object_get(obj->blob, "cause");
 	if (cause) {
 		ast_str_append(&extra, 0,
-			       "Cause: %jd\r\n",
-			       ast_json_integer_get(cause));
+			"Cause: %jd\r\n",
+			ast_json_integer_get(cause));
 	}
 
 	is_soft = ast_json_is_true(ast_json_object_get(obj->blob, "soft"));
@@ -727,9 +788,12 @@ static void channel_hangup_request_cb(void *data,
 	}
 
 	manager_event(EVENT_FLAG_CALL, manager_event,
-		      "%s%s",
-		      ast_str_buffer(channel_event_string),
-		      ast_str_buffer(extra));
+		"%s%s",
+		ast_str_buffer(channel_event_string),
+		ast_str_buffer(extra));
+
+	ast_free(channel_event_string);
+	ast_free(extra);
 }
 
 static void channel_chanspy_stop_cb(void *data, struct stasis_subscription *sub,
@@ -821,6 +885,9 @@ static void channel_dtmf_begin_cb(void *data, struct stasis_subscription *sub,
 						</enumlist>
 					</parameter>
 				</syntax>
+				<see-also>
+					<ref type="managerEvent">DTMFEnd</ref>
+				</see-also>
 		</managerEventInstance>
 	***/
 	manager_event(EVENT_FLAG_DTMF, "DTMFBegin",
@@ -867,6 +934,9 @@ static void channel_dtmf_end_cb(void *data, struct stasis_subscription *sub,
 						</enumlist>
 					</parameter>
 				</syntax>
+				<see-also>
+					<ref type="managerEvent">DTMFBegin</ref>
+				</see-also>
 		</managerEventInstance>
 	***/
 	manager_event(EVENT_FLAG_DTMF, "DTMFEnd",
@@ -893,11 +963,11 @@ static void channel_hangup_handler_cb(void *data, struct stasis_subscription *su
 		return;
 	}
 
-	if (!strcmp(action, "type")) {
+	if (!strcmp(action, "run")) {
 		event = "HangupHandlerRun";
-	} else if (!strcmp(action, "type")) {
+	} else if (!strcmp(action, "pop")) {
 		event = "HangupHandlerPop";
-	} else if (!strcmp(action, "type")) {
+	} else if (!strcmp(action, "push")) {
 		event = "HangupHandlerPush";
 	} else {
 		return;
@@ -1170,6 +1240,10 @@ int manager_channels_init(void)
 
 	ast_register_cleanup(manager_channels_shutdown);
 
+	/* The snapshot type has a special handler as it can result in multiple
+	 * manager events being queued due to aspects of the snapshot itself
+	 * changing.
+	 */
 	ret |= stasis_message_router_add_cache_update(message_router,
 		ast_channel_snapshot_type(), channel_snapshot_update, NULL);
 
@@ -1229,4 +1303,3 @@ int manager_channels_init(void)
 
 	return 0;
 }
-
