@@ -25,10 +25,11 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 400186 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/manager.h"
 #include "asterisk/app.h"
+#include "asterisk/mwi.h"
 #include "asterisk/channel.h"
 #include "asterisk/stasis_message_router.h"
 #include "asterisk/stasis.h"
@@ -176,7 +177,7 @@ int manager_mwi_init(void)
 		return -1;
 	}
 
-	ast_register_atexit(manager_mwi_shutdown);
+	ast_register_cleanup(manager_mwi_shutdown);
 
 	ret |= stasis_message_router_add(message_router,
 					 ast_mwi_state_type(),
