@@ -128,7 +128,9 @@ static void *named_acl_config_alloc(void)
 		return NULL;
 	}
 
-	if (!(cfg->named_acl_list = ao2_container_alloc(37, named_acl_hash_fn, named_acl_cmp_fn))) {
+	cfg->named_acl_list = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+		named_acl_hash_fn, NULL, named_acl_cmp_fn);
+	if (!cfg->named_acl_list) {
 		goto error;
 	}
 
@@ -436,8 +438,6 @@ int ast_named_acl_reload(void)
  */
 static void cli_display_named_acl(int fd, const char *name)
 {
-	struct ast_ha *ha;
-	int ha_index = 0;
 	int is_realtime = 0;
 
 	RAII_VAR(struct named_acl_config *, cfg, ao2_global_obj_ref(globals), ao2_cleanup);
@@ -462,12 +462,7 @@ static void cli_display_named_acl(int fd, const char *name)
 	}
 
 	ast_cli(fd, "\nACL: %s%s\n---------------------------------------------\n", name, is_realtime ? " (realtime)" : "");
-	for (ha = named_acl->ha; ha; ha = ha->next) {
-		char *addr = ast_strdupa(ast_sockaddr_stringify_addr(&ha->addr));
-		char *mask = ast_sockaddr_stringify_addr(&ha->netmask);
-		ast_cli(fd, "%3d: %s - %s/%s\n", ha_index, ha->sense == AST_SENSE_ALLOW ? "allow" : " deny", addr, mask);
-		ha_index++;
-	}
+	ast_ha_output(fd, named_acl->ha, NULL);
 }
 
 /*!
@@ -502,12 +497,10 @@ static void cli_display_named_acl_list(int fd)
 /* \brief ACL command show <name> */
 static char *handle_show_named_acl_cmd(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
-	RAII_VAR(struct named_acl_config *, cfg, ao2_global_obj_ref(globals), ao2_cleanup);
+	struct named_acl_config *cfg;
 	int length;
-	int which;
 	struct ao2_iterator i;
 	struct named_acl *named_acl;
-	char *match = NULL;
 
 	switch (cmd) {
 	case CLI_INIT:
@@ -517,23 +510,29 @@ static char *handle_show_named_acl_cmd(struct ast_cli_entry *e, int cmd, struct
 			"   Shows a list of named ACLs or lists all entries in a given named ACL.\n";
 		return NULL;
 	case CLI_GENERATE:
+		if (a->pos != 2) {
+			return NULL;
+		}
+
+		cfg = ao2_global_obj_ref(globals);
 		if (!cfg) {
 			return NULL;
 		}
 		length = strlen(a->word);
-		which = 0;
 		i = ao2_iterator_init(cfg->named_acl_list, 0);
 		while ((named_acl = ao2_iterator_next(&i))) {
-			if (!strncasecmp(a->word, named_acl->name, length) && ++which > a->n) {
-				match = ast_strdup(named_acl->name);
-				ao2_ref(named_acl, -1);
-				break;
+			if (!strncasecmp(a->word, named_acl->name, length)) {
+				if (ast_cli_completion_add(ast_strdup(named_acl->name))) {
+					ao2_ref(named_acl, -1);
+					break;
+				}
 			}
 			ao2_ref(named_acl, -1);
 		}
 		ao2_iterator_destroy(&i);
-		return match;
+		ao2_ref(cfg, -1);
 
+		return NULL;
 	}
 
 	if (a->argc == 2) {
