@@ -33,10 +33,7 @@
 ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #ifdef DEBUG_SCHEDULER
-#define DEBUG(a) do { \
-	if (option_debug) \
-		DEBUG_M(a) \
-	} while (0)
+#define DEBUG(a) a
 #else
 #define DEBUG(a)
 #endif
@@ -83,6 +80,14 @@ struct sched {
 	/*! The ID that has been popped off the scheduler context's queue */
 	struct sched_id *sched_id;
 	struct timeval when;          /*!< Absolute time event should take place */
+	/*!
+	 * \brief Tie breaker in case the when is the same for multiple entries.
+	 *
+	 * \note The oldest expiring entry in the scheduler heap goes first.
+	 * This is possible when multiple events are scheduled to expire at
+	 * the same time by internal coding.
+	 */
+	unsigned int tie_breaker;
 	int resched;                  /*!< When to reschedule */
 	int variable;                 /*!< Use return value from callback to reschedule */
 	const void *data;             /*!< Data */
@@ -107,10 +112,14 @@ struct ast_sched_context {
 	ast_mutex_t lock;
 	unsigned int eventcnt;                  /*!< Number of events processed */
 	unsigned int highwater;					/*!< highest count so far */
+	/*! Next tie breaker in case events expire at the same time. */
+	unsigned int tie_breaker;
 	struct ast_heap *sched_heap;
 	struct sched_thread *sched_thread;
 	/*! The scheduled task that is currently executing */
 	struct sched *currently_executing;
+	/*! Valid while currently_executing is not NULL */
+	pthread_t executing_thread_id;
 
 #ifdef SCHED_MAX_CACHE
 	AST_LIST_HEAD_NOLOCK(, sched) schedc;   /*!< Cache of unused schedule structures and how many */
@@ -213,9 +222,17 @@ int ast_sched_start_thread(struct ast_sched_context *con)
 	return 0;
 }
 
-static int sched_time_cmp(void *a, void *b)
+static int sched_time_cmp(void *va, void *vb)
 {
-	return ast_tvcmp(((struct sched *) b)->when, ((struct sched *) a)->when);
+	struct sched *a = va;
+	struct sched *b = vb;
+	int cmp;
+
+	cmp = ast_tvcmp(b->when, a->when);
+	if (!cmp) {
+		cmp = b->tie_breaker - a->tie_breaker;
+	}
+	return cmp;
 }
 
 struct ast_sched_context *ast_sched_context_create(void)
@@ -315,9 +332,16 @@ static int add_ids(struct ast_sched_context *con)
 		if (!new_id) {
 			break;
 		}
-		new_id->id = i;
+
+		/*
+		 * According to the API doxygen a sched ID of 0 is valid.
+		 * Unfortunately, 0 was never returned historically and
+		 * several users incorrectly coded usage of the returned
+		 * sched ID assuming that 0 was invalid.
+		 */
+		new_id->id = ++con->id_queue_size;
+
 		AST_LIST_INSERT_TAIL(&con->id_queue, new_id, list);
-		++con->id_queue_size;
 	}
 
 	return con->id_queue_size - original_size;
@@ -435,21 +459,49 @@ int ast_sched_wait(struct ast_sched_context *con)
  */
 static void schedule(struct ast_sched_context *con, struct sched *s)
 {
-	ast_heap_push(con->sched_heap, s);
+	size_t size;
 
-	if (ast_heap_size(con->sched_heap) > con->highwater) {
-		con->highwater = ast_heap_size(con->sched_heap);
+	size = ast_heap_size(con->sched_heap);
+
+	/* Record the largest the scheduler heap became for reporting purposes. */
+	if (con->highwater <= size) {
+		con->highwater = size + 1;
 	}
+
+	/* Determine the tie breaker value for the new entry. */
+	if (size) {
+		++con->tie_breaker;
+	} else {
+		/*
+		 * Restart the sequence for the first entry to make integer
+		 * roll over more unlikely.
+		 */
+		con->tie_breaker = 0;
+	}
+	s->tie_breaker = con->tie_breaker;
+
+	ast_heap_push(con->sched_heap, s);
 }
 
 /*! \brief
  * given the last event *tv and the offset in milliseconds 'when',
  * computes the next value,
  */
-static int sched_settime(struct timeval *t, int when)
+static void sched_settime(struct timeval *t, int when)
 {
 	struct timeval now = ast_tvnow();
 
+	if (when < 0) {
+		/*
+		 * A negative when value is likely a bug as it
+		 * represents a VERY large timeout time.
+		 */
+		ast_log(LOG_WARNING,
+			"Bug likely: Negative time interval %d (interpreted as %u ms) requested!\n",
+			when, (unsigned int) when);
+		ast_assert(0);
+	}
+
 	/*ast_debug(1, "TV -> %lu,%lu\n", tv->tv_sec, tv->tv_usec);*/
 	if (ast_tvzero(*t))	/* not supplied, default to now */
 		*t = now;
@@ -457,7 +509,6 @@ static int sched_settime(struct timeval *t, int when)
 	if (ast_tvcmp(*t, now) < 0) {
 		*t = now;
 	}
-	return 0;
 }
 
 int ast_sched_replace_variable(int old_id, struct ast_sched_context *con, int when, ast_sched_cb callback, const void *data, int variable)
@@ -488,17 +539,14 @@ int ast_sched_add_variable(struct ast_sched_context *con, int when, ast_sched_cb
 		tmp->variable = variable;
 		tmp->when = ast_tv(0, 0);
 		tmp->deleted = 0;
-		if (sched_settime(&tmp->when, when)) {
-			sched_release(con, tmp);
-		} else {
-			schedule(con, tmp);
-			res = tmp->sched_id->id;
-		}
+
+		sched_settime(&tmp->when, when);
+		schedule(con, tmp);
+		res = tmp->sched_id->id;
 	}
 #ifdef DUMP_SCHEDULER
 	/* Dump contents of the context while we have the lock so nothing gets screwed up by accident. */
-	if (option_debug)
-		ast_sched_dump(con);
+	ast_sched_dump(con);
 #endif
 	if (con->sched_thread) {
 		ast_cond_signal(&con->sched_thread->cond);
@@ -561,11 +609,7 @@ const void *ast_sched_find_data(struct ast_sched_context *con, int id)
  * would be two or more in the list with that
  * id.
  */
-#ifndef AST_DEVMODE
 int ast_sched_del(struct ast_sched_context *con, int id)
-#else
-int _ast_sched_del(struct ast_sched_context *con, int id, const char *file, int line, const char *function)
-#endif
 {
 	struct sched *s = NULL;
 	int *last_id = ast_threadstorage_get(&last_del_id, sizeof(int));
@@ -585,21 +629,31 @@ int _ast_sched_del(struct ast_sched_context *con, int id, const char *file, int
 		}
 		sched_release(con, s);
 	} else if (con->currently_executing && (id == con->currently_executing->sched_id->id)) {
-		s = con->currently_executing;
-		s->deleted = 1;
-		/* Wait for executing task to complete so that caller of ast_sched_del() does not
-		 * free memory out from under the task.
-		 */
-		while (con->currently_executing && (id == con->currently_executing->sched_id->id)) {
-			ast_cond_wait(&s->cond, &con->lock);
+		if (con->executing_thread_id == pthread_self()) {
+			/* The scheduled callback is trying to delete itself.
+			 * Not good as that is a deadlock. */
+			ast_log(LOG_ERROR,
+				"BUG! Trying to delete sched %d from within the callback %p.  "
+				"Ignoring so we don't deadlock\n",
+				id, con->currently_executing->callback);
+			ast_log_backtrace();
+			/* We'll return -1 below because s is NULL.
+			 * The caller will rightly assume that the unscheduling failed. */
+		} else {
+			s = con->currently_executing;
+			s->deleted = 1;
+			/* Wait for executing task to complete so that the caller of
+			 * ast_sched_del() does not free memory out from under the task. */
+			while (con->currently_executing && (id == con->currently_executing->sched_id->id)) {
+				ast_cond_wait(&s->cond, &con->lock);
+			}
+			/* Do not sched_release() here because ast_sched_runq() will do it */
 		}
-		/* Do not sched_release() here because ast_sched_runq() will do it */
 	}
 
 #ifdef DUMP_SCHEDULER
 	/* Dump contents of the context while we have the lock so nothing gets screwed up by accident. */
-	if (option_debug)
-		ast_sched_dump(con);
+	ast_sched_dump(con);
 #endif
 	if (con->sched_thread) {
 		ast_cond_signal(&con->sched_thread->cond);
@@ -660,25 +714,33 @@ void ast_sched_report(struct ast_sched_context *con, struct ast_str **buf, struc
 void ast_sched_dump(struct ast_sched_context *con)
 {
 	struct sched *q;
-	struct timeval when = ast_tvnow();
+	struct timeval when;
 	int x;
 	size_t heap_size;
+
+	if (!DEBUG_ATLEAST(1)) {
+		return;
+	}
+
+	when = ast_tvnow();
 #ifdef SCHED_MAX_CACHE
-	ast_debug(1, "Asterisk Schedule Dump (%zu in Q, %u Total, %u Cache, %u high-water)\n", ast_heap_size(con->sched_heap), con->eventcnt - 1, con->schedccnt, con->highwater);
+	ast_log(LOG_DEBUG, "Asterisk Schedule Dump (%zu in Q, %u Total, %u Cache, %u high-water)\n",
+		ast_heap_size(con->sched_heap), con->eventcnt - 1, con->schedccnt, con->highwater);
 #else
-	ast_debug(1, "Asterisk Schedule Dump (%zu in Q, %u Total, %u high-water)\n", ast_heap_size(con->sched_heap), con->eventcnt - 1, con->highwater);
+	ast_log(LOG_DEBUG, "Asterisk Schedule Dump (%zu in Q, %u Total, %u high-water)\n",
+		ast_heap_size(con->sched_heap), con->eventcnt - 1, con->highwater);
 #endif
 
-	ast_debug(1, "=============================================================\n");
-	ast_debug(1, "|ID    Callback          Data              Time  (sec:ms)   |\n");
-	ast_debug(1, "+-----+-----------------+-----------------+-----------------+\n");
+	ast_log(LOG_DEBUG, "=============================================================\n");
+	ast_log(LOG_DEBUG, "|ID    Callback          Data              Time  (sec:ms)   |\n");
+	ast_log(LOG_DEBUG, "+-----+-----------------+-----------------+-----------------+\n");
 	ast_mutex_lock(&con->lock);
 	heap_size = ast_heap_size(con->sched_heap);
 	for (x = 1; x <= heap_size; x++) {
 		struct timeval delta;
 		q = ast_heap_peek(con->sched_heap, x);
 		delta = ast_tvsub(q->when, when);
-		ast_debug(1, "|%.4d | %-15p | %-15p | %.6ld : %.6ld |\n",
+		ast_log(LOG_DEBUG, "|%.4d | %-15p | %-15p | %.6ld : %.6ld |\n",
 			q->sched_id->id,
 			q->callback,
 			q->data,
@@ -686,7 +748,7 @@ void ast_sched_dump(struct ast_sched_context *con)
 			(long int)delta.tv_usec);
 	}
 	ast_mutex_unlock(&con->lock);
-	ast_debug(1, "=============================================================\n");
+	ast_log(LOG_DEBUG, "=============================================================\n");
 }
 
 /*! \brief
@@ -726,6 +788,7 @@ int ast_sched_runq(struct ast_sched_context *con)
 		 */
 
 		con->currently_executing = current;
+		con->executing_thread_id = pthread_self();
 		ast_mutex_unlock(&con->lock);
 		res = current->callback(current->data);
 		ast_mutex_lock(&con->lock);
@@ -737,11 +800,8 @@ int ast_sched_runq(struct ast_sched_context *con)
 			 * If they return non-zero, we should schedule them to be
 			 * run again.
 			 */
-			if (sched_settime(&current->when, current->variable? res : current->resched)) {
-				sched_release(con, current);
-			} else {
-				schedule(con, current);
-			}
+			sched_settime(&current->when, current->variable ? res : current->resched);
+			schedule(con, current);
 		} else {
 			/* No longer needed, so release it */
 			sched_release(con, current);
