@@ -314,7 +314,7 @@
 				<parameter name="ReceivedChallenge" required="false">
 					<para>The challenge that was received.</para>
 				</parameter>
-				<parameter name="RecievedHash" required="false">
+				<parameter name="ReceivedHash" required="false">
 					<para>The hash that was received.</para>
 				</parameter>
 			</syntax>
@@ -362,7 +362,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 420124 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/utils.h"
 #include "asterisk/strings.h"
@@ -374,7 +374,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 420124 $")
 #include "asterisk/json.h"
 #include "asterisk/astobj2.h"
 
-static const size_t TIMESTAMP_STR_LEN = 32;
 static const size_t SECURITY_EVENT_BUF_INIT_LEN = 256;
 
 /*! \brief Security Topic */
@@ -429,7 +428,7 @@ static struct ast_manager_event_blob *security_event_to_ami_blob(struct ast_json
 	event_type_json = ast_json_object_get(json, "SecurityEvent");
 	event_type = ast_json_integer_get(event_type_json);
 
-	ast_assert(event_type >= 0 && event_type < AST_SECURITY_EVENT_NUM_TYPES);
+	ast_assert((unsigned int)event_type < AST_SECURITY_EVENT_NUM_TYPES);
 
 	if (!(str = ast_str_create(SECURITY_EVENT_BUF_INIT_LEN))) {
 		return NULL;
@@ -487,7 +486,7 @@ int ast_security_stasis_init(void)
 {
 	ast_register_cleanup(security_stasis_cleanup);
 
-	security_topic = stasis_topic_create("ast_security");
+	security_topic = stasis_topic_create("security:all");
 	if (!security_topic) {
 		return -1;
 	}
@@ -887,7 +886,7 @@ const char *ast_security_event_severity_get_name(
 
 static int check_event_type(const enum ast_security_event_type event_type)
 {
-	if (event_type < 0 || event_type >= AST_SECURITY_EVENT_NUM_TYPES) {
+	if ((unsigned int)event_type >= AST_SECURITY_EVENT_NUM_TYPES) {
 		ast_log(LOG_ERROR, "Invalid security event type %u\n", event_type);
 		return -1;
 	}
@@ -1173,7 +1172,7 @@ return_error:
 
 int ast_security_event_report(const struct ast_security_event_common *sec)
 {
-	if (sec->event_type < 0 || sec->event_type >= AST_SECURITY_EVENT_NUM_TYPES) {
+	if ((unsigned int)sec->event_type >= AST_SECURITY_EVENT_NUM_TYPES) {
 		ast_log(LOG_ERROR, "Invalid security event type\n");
 		return -1;
 	}
@@ -1197,5 +1196,3 @@ int ast_security_event_report(const struct ast_security_event_common *sec)
 
 	return 0;
 }
-
-
