@@ -67,31 +67,36 @@ static void all_dtor(void *obj)
 struct stasis_cp_all *stasis_cp_all_create(const char *name,
 	snapshot_get_id id_fn)
 {
-	RAII_VAR(char *, cached_name, NULL, ast_free);
-	RAII_VAR(struct stasis_cp_all *, all, NULL, ao2_cleanup);
+	char *cached_name = NULL;
+	struct stasis_cp_all *all;
+	static int cache_id;
 
 	all = ao2_t_alloc(sizeof(*all), all_dtor, name);
 	if (!all) {
 		return NULL;
 	}
 
-	ast_asprintf(&cached_name, "%s-cached", name);
+	ast_asprintf(&cached_name, "cache_pattern:%d/%s", ast_atomic_fetchadd_int(&cache_id, +1), name);
 	if (!cached_name) {
+		ao2_ref(all, -1);
+
 		return NULL;
 	}
 
 	all->topic = stasis_topic_create(name);
 	all->topic_cached = stasis_topic_create(cached_name);
+	ast_free(cached_name);
 	all->cache = stasis_cache_create(id_fn);
 	all->forward_all_to_cached =
 		stasis_forward_all(all->topic, all->topic_cached);
 
 	if (!all->topic || !all->topic_cached || !all->cache ||
 		!all->forward_all_to_cached) {
+		ao2_ref(all, -1);
+
 		return NULL;
 	}
 
-	ao2_ref(all, +1);
 	return all;
 }
 
@@ -136,7 +141,30 @@ static void one_dtor(void *obj)
 struct stasis_cp_single *stasis_cp_single_create(struct stasis_cp_all *all,
 	const char *name)
 {
-	RAII_VAR(struct stasis_cp_single *, one, NULL, ao2_cleanup);
+	struct stasis_cp_single *one;
+
+	one = stasis_cp_sink_create(all, name);
+	if (!one) {
+		return NULL;
+	}
+
+	one->forward_topic_to_all = stasis_forward_all(one->topic, all->topic);
+	one->forward_cached_to_all = stasis_forward_all(
+		stasis_caching_get_topic(one->topic_cached), all->topic_cached);
+
+	if (!one->forward_topic_to_all || !one->forward_cached_to_all) {
+		ao2_ref(one, -1);
+
+		return NULL;
+	}
+
+	return one;
+}
+
+struct stasis_cp_single *stasis_cp_sink_create(struct stasis_cp_all *all,
+	const char *name)
+{
+	struct stasis_cp_single *one;
 
 	one = ao2_t_alloc(sizeof(*one), one_dtor, name);
 	if (!one) {
@@ -145,24 +173,18 @@ struct stasis_cp_single *stasis_cp_single_create(struct stasis_cp_all *all,
 
 	one->topic = stasis_topic_create(name);
 	if (!one->topic) {
+		ao2_ref(one, -1);
+
 		return NULL;
 	}
+
 	one->topic_cached = stasis_caching_topic_create(one->topic, all->cache);
 	if (!one->topic_cached) {
+		ao2_ref(one, -1);
+
 		return NULL;
 	}
 
-	one->forward_topic_to_all = stasis_forward_all(one->topic, all->topic);
-	if (!one->forward_topic_to_all) {
-		return NULL;
-	}
-	one->forward_cached_to_all = stasis_forward_all(
-		stasis_caching_get_topic(one->topic_cached), all->topic_cached);
-	if (!one->forward_cached_to_all) {
-		return NULL;
-	}
-
-	ao2_ref(one, +1);
 	return one;
 }
 
@@ -199,3 +221,20 @@ struct stasis_topic *stasis_cp_single_topic_cached(
 	return stasis_caching_get_topic(one->topic_cached);
 }
 
+int stasis_cp_single_accept_message_type(struct stasis_cp_single *one,
+	struct stasis_message_type *type)
+{
+	if (!one) {
+		return -1;
+	}
+	return stasis_caching_accept_message_type(one->topic_cached, type);
+}
+
+int stasis_cp_single_set_filter(struct stasis_cp_single *one,
+	enum stasis_subscription_message_filter filter)
+{
+	if (!one) {
+		return -1;
+	}
+	return stasis_caching_set_filter(one->topic_cached, filter);
+}
