@@ -21,13 +21,14 @@
  */
 
 /*** MODULEINFO
+	<depend>res_calendar</depend>
 	<depend>neon29</depend>
 	<support_level>core</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 425289 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include <ne_request.h>
 #include <ne_session.h>
@@ -119,6 +120,7 @@ static void ewscal_destructor(void *obj)
 	if (pvt->session) {
 		ne_session_destroy(pvt->session);
 	}
+	ne_uri_free(&pvt->uri);
 	ast_string_field_free_memory(pvt);
 
 	ao2_callback(pvt->events, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL);
