@@ -33,7 +33,7 @@
  * \addtogroup configuration_file Configuration Files
  */
 
-/*! 
+/*!
  * \page res_config_sqlite3.conf res_config_sqlite3.conf
  * \verbinclude res_config_sqlite3.conf.sample
  */
@@ -104,6 +104,7 @@ struct realtime_sqlite3_db {
 	unsigned int debug:1;
 	unsigned int exiting:1;
 	unsigned int wakeup:1;
+	unsigned int has_batch_thread:1;
 	unsigned int batch;
 };
 
@@ -368,13 +369,14 @@ void db_start_batch(struct realtime_sqlite3_db *db)
 	if (db->batch) {
 		ast_cond_init(&db->cond, NULL);
 		ao2_ref(db, +1);
-		ast_pthread_create_background(&db->syncthread, NULL, db_sync_thread, db);
+		db->has_batch_thread = !ast_pthread_create_background(&db->syncthread, NULL, db_sync_thread, db);
 	}
 }
 
 void db_stop_batch(struct realtime_sqlite3_db *db)
 {
-	if (db->batch) {
+	if (db->has_batch_thread) {
+		db->has_batch_thread = 0;
 		db->exiting = 1;
 		db_sync(db);
 		pthread_join(db->syncthread, NULL);
@@ -1171,7 +1173,7 @@ static int realtime_sqlite3_require(const char *database, const char *table, va_
 	struct realtime_sqlite3_db *db;
 
 	/* SQLite3 columns are dynamically typed, with type affinity. Built-in functions will
-	 * return the results as char * anyway. The only field that that cannot contain text
+	 * return the results as char * anyway. The only field that cannot contain text
 	 * data is an INTEGER PRIMARY KEY, which must be a 64-bit signed integer. So, for
 	 * the purposes here we really only care whether the column exists and not what its
 	 * type or length is. */
@@ -1185,7 +1187,9 @@ static int realtime_sqlite3_require(const char *database, const char *table, va_
 		return -1;
 	}
 
-	if (!(columns = ao2_container_alloc(31, str_hash_fn, str_cmp_fn))) {
+	columns = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 31,
+		str_hash_fn, NULL, str_cmp_fn);
+	if (!columns) {
 		unref_db(&db);
 	   return -1;
 	}
@@ -1352,27 +1356,29 @@ static void discover_sqlite3_caps(void)
  * Module loading including tests for configuration or dependencies.
  * This function can return AST_MODULE_LOAD_FAILURE, AST_MODULE_LOAD_DECLINE,
  * or AST_MODULE_LOAD_SUCCESS. If a dependency or environment variable fails
- * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the 
- * configuration file or other non-critical problem return 
+ * tests return AST_MODULE_LOAD_FAILURE. If the module can not load the
+ * configuration file or other non-critical problem return
  * AST_MODULE_LOAD_DECLINE. On success return AST_MODULE_LOAD_SUCCESS.
  */
 static int load_module(void)
 {
 	discover_sqlite3_caps();
 
-	if (!((databases = ao2_container_alloc(DB_BUCKETS, db_hash_fn, db_cmp_fn)))) {
-		return AST_MODULE_LOAD_FAILURE;
+	databases = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, DB_BUCKETS,
+		db_hash_fn, NULL, db_cmp_fn);
+	if (!databases) {
+		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	if (parse_config(0)) {
 		ao2_ref(databases, -1);
-		return AST_MODULE_LOAD_FAILURE;
+		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	if (!(ast_config_engine_register(&sqlite3_config_engine))) {
 		ast_log(LOG_ERROR, "The config API must have changed, this shouldn't happen.\n");
 		ao2_ref(databases, -1);
-		return AST_MODULE_LOAD_FAILURE;
+		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	return AST_MODULE_LOAD_SUCCESS;
