@@ -170,12 +170,8 @@ struct mohclass {
 	char announcement[256];
 	char mode[80];
 	char digit;
-	/*! A dynamically sized array to hold the list of filenames in "files" mode */
-	char **filearray;
-	/*! The current size of the filearray */
-	int allowed_files;
-	/*! The current number of files loaded into the filearray */
-	int total_files;
+	/*! An immutable vector of filenames in "files" mode */
+	struct ast_vector_string *files;
 	unsigned int flags;
 	/*! The format from the MOH source, not applicable to "files" mode */
 	struct ast_format *format;
@@ -315,7 +311,9 @@ static void moh_files_release(struct ast_channel *chan, void *data)
 static int ast_moh_files_next(struct ast_channel *chan)
 {
 	struct moh_files_state *state = ast_channel_music_state(chan);
+	struct ast_vector_string *files;
 	int tries;
+	size_t file_count;
 
 	/* Discontinue a stream if it is running already */
 	if (ast_channel_stream(chan)) {
@@ -331,26 +329,31 @@ static int ast_moh_files_next(struct ast_channel *chan)
 		}
 	} else {
 		state->announcement = 0;
-		state->samples = 0;
 	}
 
-	if (!state->class->total_files) {
+	ao2_lock(state->class);
+	files = ao2_bump(state->class->files);
+	ao2_unlock(state->class);
+
+	file_count = AST_VECTOR_SIZE(files);
+	if (!file_count) {
 		ast_log(LOG_WARNING, "No files available for class '%s'\n", state->class->name);
+		ao2_ref(files, -1);
 		return -1;
 	}
 
 	if (state->pos == 0 && ast_strlen_zero(state->save_pos_filename)) {
 		/* First time so lets play the file. */
 		state->save_pos = -1;
-	} else if (state->save_pos >= 0 && state->save_pos < state->class->total_files && !strcmp(state->class->filearray[state->save_pos], state->save_pos_filename)) {
+	} else if (state->save_pos >= 0 && state->save_pos < file_count && !strcmp(AST_VECTOR_GET(files, state->save_pos), state->save_pos_filename)) {
 		/* If a specific file has been saved confirm it still exists and that it is still valid */
 		state->pos = state->save_pos;
 		state->save_pos = -1;
 	} else if (ast_test_flag(state->class, MOH_RANDOMIZE)) {
 		/* Get a random file and ensure we can open it */
 		for (tries = 0; tries < 20; tries++) {
-			state->pos = ast_random() % state->class->total_files;
-			if (ast_fileexists(state->class->filearray[state->pos], NULL, NULL) > 0) {
+			state->pos = ast_random() % file_count;
+			if (ast_fileexists(AST_VECTOR_GET(files, state->pos), NULL, NULL) > 0) {
 				break;
 			}
 		}
@@ -359,29 +362,30 @@ static int ast_moh_files_next(struct ast_channel *chan)
 	} else {
 		/* This is easy, just increment our position and make sure we don't exceed the total file count */
 		state->pos++;
-		state->pos %= state->class->total_files;
+		state->pos %= file_count;
 		state->save_pos = -1;
 		state->samples = 0;
 	}
 
-	for (tries = 0; tries < state->class->total_files; ++tries) {
-		if (ast_openstream_full(chan, state->class->filearray[state->pos], ast_channel_language(chan), 1)) {
+	for (tries = 0; tries < file_count; ++tries) {
+		if (ast_openstream_full(chan, AST_VECTOR_GET(files, state->pos), ast_channel_language(chan), 1)) {
 			break;
 		}
 
-		ast_log(LOG_WARNING, "Unable to open file '%s': %s\n", state->class->filearray[state->pos], strerror(errno));
+		ast_log(LOG_WARNING, "Unable to open file '%s': %s\n", AST_VECTOR_GET(files, state->pos), strerror(errno));
 		state->pos++;
-		state->pos %= state->class->total_files;
+		state->pos %= file_count;
 	}
 
-	if (tries == state->class->total_files) {
+	if (tries == file_count) {
+		ao2_ref(files, -1);
 		return -1;
 	}
 
 	/* Record the pointer to the filename for position resuming later */
-	ast_copy_string(state->save_pos_filename, state->class->filearray[state->pos], sizeof(state->save_pos_filename));
+	ast_copy_string(state->save_pos_filename, AST_VECTOR_GET(files, state->pos), sizeof(state->save_pos_filename));
 
-	ast_debug(1, "%s Opened file %d '%s'\n", ast_channel_name(chan), state->pos, state->class->filearray[state->pos]);
+	ast_debug(1, "%s Opened file %d '%s'\n", ast_channel_name(chan), state->pos, state->save_pos_filename);
 
 	if (state->samples) {
 		size_t loc;
@@ -396,16 +400,34 @@ static int ast_moh_files_next(struct ast_channel *chan)
 		}
 	}
 
+	ao2_ref(files, -1);
 	return 0;
 }
 
 static struct ast_frame *moh_files_readframe(struct ast_channel *chan)
 {
-	struct ast_frame *f = NULL;
+	struct ast_frame *f;
 
-	if (!(ast_channel_stream(chan) && (f = ast_readframe(ast_channel_stream(chan))))) {
-		if (!ast_moh_files_next(chan))
+	f = ast_readframe(ast_channel_stream(chan));
+	if (!f) {
+		/* Either there was no file stream setup or we reached EOF. */
+		if (!ast_moh_files_next(chan)) {
+			/*
+			 * Either we resetup the previously saved file stream position
+			 * or we started a new file stream.
+			 */
 			f = ast_readframe(ast_channel_stream(chan));
+			if (!f) {
+				/*
+				 * We can get here if we were very unlucky because the
+				 * resetup file stream was saved at EOF when MOH was
+				 * previously stopped.
+				 */
+				if (!ast_moh_files_next(chan)) {
+					f = ast_readframe(ast_channel_stream(chan));
+				}
+			}
+		}
 	}
 
 	return f;
@@ -453,7 +475,12 @@ static int moh_files_generator(struct ast_channel *chan, void *data, int len, in
 			return -1;
 		}
 
-		state->samples += f->samples;
+		/* Only track our offset within the current file if we are not in the
+		 * the middle of an announcement */
+		if (!state->announcement) {
+			state->samples += f->samples;
+		}
+
 		state->sample_queue -= f->samples;
 		if (ast_format_cmp(f->subclass.format, state->mohwfmt) == AST_FORMAT_CMP_NOT_EQUAL) {
 			ao2_replace(state->mohwfmt, f->subclass.format);
@@ -472,6 +499,7 @@ static void *moh_files_alloc(struct ast_channel *chan, void *params)
 {
 	struct moh_files_state *state;
 	struct mohclass *class = params;
+	size_t file_count;
 
 	state = ast_channel_music_state(chan);
 	if (!state && (state = ast_calloc(1, sizeof(*state)))) {
@@ -487,14 +515,18 @@ static void *moh_files_alloc(struct ast_channel *chan, void *params)
 		}
 	}
 
+	ao2_lock(class);
+	file_count = AST_VECTOR_SIZE(class->files);
+	ao2_unlock(class);
+
 	/* Resume MOH from where we left off last time or start from scratch? */
-	if (state->save_total != class->total_files || strcmp(state->name, class->name) != 0) {
+	if (state->save_total != file_count || strcmp(state->name, class->name) != 0) {
 		/* Start MOH from scratch. */
 		ao2_cleanup(state->origwfmt);
 		ao2_cleanup(state->mohwfmt);
 		memset(state, 0, sizeof(*state));
-		if (ast_test_flag(class, MOH_RANDOMIZE) && class->total_files) {
-			state->pos = ast_random() % class->total_files;
+		if (ast_test_flag(class, MOH_RANDOMIZE) && file_count) {
+			state->pos = ast_random() % file_count;
 		}
 	}
 
@@ -504,7 +536,7 @@ static void *moh_files_alloc(struct ast_channel *chan, void *params)
 	ao2_replace(state->mohwfmt, ast_channel_writeformat(chan));
 	/* For comparison on restart of MOH (see above) */
 	ast_copy_string(state->name, class->name, sizeof(state->name));
-	state->save_total = class->total_files;
+	state->save_total = file_count;
 
 	moh_post_start(chan, class->name);
 
@@ -929,16 +961,12 @@ static struct mohdata *mohalloc(struct mohclass *cl)
 	if (!(moh = ast_calloc(1, sizeof(*moh))))
 		return NULL;
 
-	if (pipe(moh->pipe)) {
+	if (ast_pipe_nonblock(moh->pipe)) {
 		ast_log(LOG_WARNING, "Failed to create pipe: %s\n", strerror(errno));
 		ast_free(moh);
 		return NULL;
 	}
 
-	/* Make entirely non-blocking */
-	ast_fd_set_flags(moh->pipe[0], O_NONBLOCK);
-	ast_fd_set_flags(moh->pipe[1], O_NONBLOCK);
-
 	moh->f.frametype = AST_FRAME_VOICE;
 	moh->f.subclass.format = cl->format;
 	moh->f.offset = AST_FRIENDLY_OFFSET;
@@ -1063,54 +1091,35 @@ static struct ast_generator mohgen = {
 	.digit    = moh_handle_digit,
 };
 
-static int moh_add_file(struct mohclass *class, const char *filepath)
+static void moh_file_vector_destructor(void *obj)
 {
-	if (!class->allowed_files) {
-		class->filearray = ast_calloc(1, INITIAL_NUM_FILES * sizeof(*class->filearray));
-		if (!class->filearray) {
-			return -1;
-		}
-		class->allowed_files = INITIAL_NUM_FILES;
-	} else if (class->total_files == class->allowed_files) {
-		char **new_array;
-
-		new_array = ast_realloc(class->filearray, class->allowed_files * sizeof(*class->filearray) * 2);
-		if (!new_array) {
-			return -1;
-		}
-		class->filearray = new_array;
-		class->allowed_files *= 2;
-	}
-
-	class->filearray[class->total_files] = ast_strdup(filepath);
-	if (!class->filearray[class->total_files]) {
-		return -1;
-	}
-
-	class->total_files++;
-
-	return 0;
+	struct ast_vector_string *files = obj;
+	AST_VECTOR_RESET(files, ast_free);
+	AST_VECTOR_FREE(files);
 }
 
-static int moh_sort_compare(const void *i1, const void *i2)
+static struct ast_vector_string *moh_file_vector_alloc(int initial_capacity)
 {
-	char *s1, *s2;
-
-	s1 = ((char **)i1)[0];
-	s2 = ((char **)i2)[0];
-
-	return strcasecmp(s1, s2);
+	struct ast_vector_string *files = ao2_alloc_options(
+		sizeof(struct ast_vector_string),
+		moh_file_vector_destructor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (files) {
+		AST_VECTOR_INIT(files, initial_capacity);
+	}
+	return files;
 }
 
 static int moh_scan_files(struct mohclass *class) {
 
 	DIR *files_DIR;
 	struct dirent *files_dirent;
-	char dir_path[PATH_MAX];
+	char dir_path[PATH_MAX - sizeof(class->dir)];
 	char filepath[PATH_MAX];
 	char *ext;
 	struct stat statbuf;
-	int i;
+	int res;
+	struct ast_vector_string *files;
 
 	if (class->dir[0] != '/') {
 		snprintf(dir_path, sizeof(dir_path), "%s/%s", ast_config_AST_DATA_DIR, class->dir);
@@ -1124,12 +1133,15 @@ static int moh_scan_files(struct mohclass *class) {
 		return -1;
 	}
 
-	for (i = 0; i < class->total_files; i++) {
-		ast_free(class->filearray[i]);
+	files = moh_file_vector_alloc(16); /* 16 seems like a reasonable default */
+	if (!files) {
+		closedir(files_DIR);
+		return -1;
 	}
-	class->total_files = 0;
 
 	while ((files_dirent = readdir(files_DIR))) {
+		char *filepath_copy;
+
 		/* The file name must be at least long enough to have the file type extension */
 		if ((strlen(files_dirent->d_name) < 4))
 			continue;
@@ -1154,20 +1166,36 @@ static int moh_scan_files(struct mohclass *class) {
 			*ext = '\0';
 
 		/* if the file is present in multiple formats, ensure we only put it into the list once */
-		for (i = 0; i < class->total_files; i++)
-			if (!strcmp(filepath, class->filearray[i]))
-				break;
+		if (AST_VECTOR_GET_CMP(files, &filepath[0], !strcmp)) {
+			continue;
+		}
 
-		if (i == class->total_files) {
-			if (moh_add_file(class, filepath))
-				break;
+		filepath_copy = ast_strdup(filepath);
+		if (!filepath_copy) {
+			break;
+		}
+
+		if (ast_test_flag(class, MOH_SORTALPHA)) {
+			res = AST_VECTOR_ADD_SORTED(files, filepath_copy, strcasecmp);
+		} else {
+			res = AST_VECTOR_APPEND(files, filepath_copy);
+		}
+
+		if (res) {
+			ast_free(filepath_copy);
+			break;
 		}
 	}
 
 	closedir(files_DIR);
-	if (ast_test_flag(class, MOH_SORTALPHA))
-		qsort(&class->filearray[0], class->total_files, sizeof(char *), moh_sort_compare);
-	return class->total_files;
+
+	AST_VECTOR_COMPACT(files);
+
+	ao2_lock(class);
+	ao2_replace(class->files, files);
+	ao2_unlock(class);
+
+	return AST_VECTOR_SIZE(files);
 }
 
 static int init_files_class(struct mohclass *class)
@@ -1318,6 +1346,13 @@ static int _moh_register(struct mohclass *moh, int reload, int unref, const char
 	return 0;
 }
 
+#define moh_unregister(a) _moh_unregister(a,__FILE__,__LINE__,__PRETTY_FUNCTION__)
+static int _moh_unregister(struct mohclass *moh, const char *file, int line, const char *funcname)
+{
+	ao2_t_unlink(mohclasses, moh, "Removing class from container");
+	return 0;
+}
+
 static void local_ast_moh_cleanup(struct ast_channel *chan)
 {
 	struct moh_files_state *state = ast_channel_music_state(chan);
@@ -1338,6 +1373,82 @@ static void local_ast_moh_cleanup(struct ast_channel *chan)
 	}
 }
 
+/*! \brief Support routing for 'moh unregister class' CLI
+ * This is in charge of generating all strings that match a prefix in the
+ * given position. As many functions of this kind, each invokation has
+ * O(state) time complexity so be careful in using it.
+ */
+static char *complete_mohclass_realtime(const char *line, const char *word, int pos, int state)
+{
+	int which=0;
+	struct mohclass *cur;
+	char *c = NULL;
+	int wordlen = strlen(word);
+	struct ao2_iterator i;
+
+	if (pos != 3) {
+		return NULL;
+	}
+
+	i = ao2_iterator_init(mohclasses, 0);
+	while ((cur = ao2_t_iterator_next(&i, "iterate thru mohclasses"))) {
+		if (cur->realtime && !strncasecmp(cur->name, word, wordlen) && ++which > state) {
+			c = ast_strdup(cur->name);
+			mohclass_unref(cur, "drop ref in iterator loop break");
+			break;
+		}
+		mohclass_unref(cur, "drop ref in iterator loop");
+	}
+	ao2_iterator_destroy(&i);
+
+	return c;
+}
+
+static char *handle_cli_moh_unregister_class(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	struct mohclass *cur;
+	int len;
+	int found = 0;
+	struct ao2_iterator i;
+
+	switch (cmd) {
+		case CLI_INIT:
+			e->command = "moh unregister class";
+			e->usage =
+				"Usage: moh unregister class <class>\n"
+				"       Unregisters a realtime moh class.\n";
+			return NULL;
+		case CLI_GENERATE:
+			return complete_mohclass_realtime(a->line, a->word, a->pos, a->n);
+	}
+
+	if (a->argc != 4)
+		return CLI_SHOWUSAGE;
+
+	len = strlen(a->argv[3]);
+
+	i = ao2_iterator_init(mohclasses, 0);
+	while ((cur = ao2_t_iterator_next(&i, "iterate thru mohclasses"))) {
+		if (cur->realtime && len == strlen(cur->name) && !strncasecmp(cur->name, a->argv[3], len)) {
+			found = 1;
+			break;
+		}
+		mohclass_unref(cur, "drop ref in iterator loop");
+	}
+	ao2_iterator_destroy(&i);
+
+	if (found) {
+		moh_unregister(cur);
+		mohclass_unref(cur, "drop ref after unregister");
+	} else {
+		ast_cli(a->fd, "No such realtime moh class '%s'\n", a->argv[3]);
+	}
+
+	return CLI_SUCCESS;
+}
+
+
+
 static void moh_class_destructor(void *obj);
 
 #define moh_class_malloc()	_moh_class_malloc(__FILE__,__LINE__,__PRETTY_FUNCTION__)
@@ -1360,6 +1471,13 @@ static struct mohclass *_moh_class_malloc(const char *file, int line, const char
 		class->format = ao2_bump(ast_format_slin);
 		class->srcfd = -1;
 		class->kill_delay = 100000;
+
+		/* We create an empty one by default */
+		class->files = moh_file_vector_alloc(0);
+		if (!class->files) {
+			ao2_ref(class, -1);
+			return NULL;
+		}
 	}
 
 	return class;
@@ -1448,7 +1566,7 @@ static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, con
 					ast_set_flag(mohclass, MOH_RANDOMIZE);
 				else if (!strcasecmp(tmp->name, "sort") && !strcasecmp(tmp->value, "alpha"))
 					ast_set_flag(mohclass, MOH_SORTALPHA);
-				else if (!strcasecmp(tmp->name, "format")) {
+				else if (!strcasecmp(tmp->name, "format") && !ast_strlen_zero(tmp->value)) {
 					ao2_cleanup(mohclass->format);
 					mohclass->format = ast_format_cache_get(tmp->value);
 					if (!mohclass->format) {
@@ -1501,6 +1619,14 @@ static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, con
 				mohclass->start -= respawn_time;
 
 				if (!strcasecmp(mohclass->mode, "files")) {
+					/*
+					 * XXX moh_scan_files returns -1 if it is unable to open the
+					 * configured directory or there is a memory allocation
+					 * failure. Otherwise it returns the number of files for this music
+					 * class. This check is only checking if the number of files is zero
+					 * and it ignores the -1 case. To avoid a behavior change we keep this
+					 * as-is, but we should address what the 'correct' behavior should be.
+					 */
 					if (!moh_scan_files(mohclass)) {
 						mohclass = mohclass_unref(mohclass, "unreffing potential mohclass (moh_scan_files failed)");
 						return -1;
@@ -1566,6 +1692,13 @@ static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, con
 
 	/* If we are using a cached realtime class with files, re-scan the files */
 	if (!var && ast_test_flag(global_flags, MOH_CACHERTCLASSES) && mohclass->realtime && !strcasecmp(mohclass->mode, "files")) {
+		/*
+		 * XXX moh_scan_files returns -1 if it is unable to open the configured directory
+		 * or there is a memory allocation failure. Otherwise it returns the number of
+		 * files for this music class. This check is only checking if the number of files
+		 * is zero and it ignores the -1 case. To avoid a behavior change we keep this
+		 * as-is, but we should address what the 'correct' behavior should be.
+		 */
 		if (!moh_scan_files(mohclass)) {
 			mohclass = mohclass_unref(mohclass, "unreffing potential mohclass (moh_scan_files failed)");
 			return -1;
@@ -1573,7 +1706,13 @@ static int local_ast_moh_start(struct ast_channel *chan, const char *mclass, con
 	}
 
 	if (!state || !state->class || strcmp(mohclass->name, state->class->name)) {
-		if (mohclass->total_files) {
+		size_t file_count;
+
+		ao2_lock(mohclass);
+		file_count = AST_VECTOR_SIZE(mohclass->files);
+		ao2_unlock(mohclass);
+
+		if (file_count) {
 			res = ast_activate_generator(chan, &moh_file_stream, mohclass);
 		} else {
 			res = ast_activate_generator(chan, &mohgen, mohclass);
@@ -1618,6 +1757,7 @@ static void moh_class_destructor(void *obj)
 	while ((member = AST_LIST_REMOVE_HEAD(&class->members, list))) {
 		ast_free(member);
 	}
+	ao2_cleanup(class->files);
 	ao2_unlock(class);
 
 	/* Kill the thread first, so it cannot restart the child process while the
@@ -1652,15 +1792,6 @@ static void moh_class_destructor(void *obj)
 		class->srcfd = -1;
 	}
 
-	if (class->filearray) {
-		int i;
-		for (i = 0; i < class->total_files; i++) {
-			ast_free(class->filearray[i]);
-		}
-		ast_free(class->filearray);
-		class->filearray = NULL;
-	}
-
 	if (class->timer) {
 		ast_timer_close(class->timer);
 		class->timer = NULL;
@@ -1768,7 +1899,7 @@ static int load_moh_classes(int reload)
 				ast_set_flag(class, MOH_RANDOMIZE);
 			} else if (!strcasecmp(var->name, "sort") && !strcasecmp(var->value, "alpha")) {
 				ast_set_flag(class, MOH_SORTALPHA);
-			} else if (!strcasecmp(var->name, "format")) {
+			} else if (!strcasecmp(var->name, "format") && !ast_strlen_zero(var->value)) {
 				ao2_cleanup(class->format);
 				class->format = ast_format_cache_get(var->value);
 				if (!class->format) {
@@ -1883,16 +2014,21 @@ static char *handle_cli_moh_show_files(struct ast_cli_entry *e, int cmd, struct
 
 	i = ao2_iterator_init(mohclasses, 0);
 	for (; (class = ao2_t_iterator_next(&i, "Show files iterator")); mohclass_unref(class, "Unref iterator in moh show files")) {
-		int x;
+		struct ast_vector_string *files;
 
-		if (!class->total_files) {
-			continue;
+		ao2_lock(class);
+		files = ao2_bump(class->files);
+		ao2_unlock(class);
+
+		if (AST_VECTOR_SIZE(files)) {
+			int x;
+			ast_cli(a->fd, "Class: %s\n", class->name);
+			for (x = 0; x < AST_VECTOR_SIZE(files); x++) {
+				ast_cli(a->fd, "\tFile: %s\n", AST_VECTOR_GET(files, x));
+			}
 		}
 
-		ast_cli(a->fd, "Class: %s\n", class->name);
-		for (x = 0; x < class->total_files; x++) {
-			ast_cli(a->fd, "\tFile: %s\n", class->filearray[x]);
-		}
+		ao2_ref(files, -1);
 	}
 	ao2_iterator_destroy(&i);
 
@@ -1942,9 +2078,10 @@ static char *handle_cli_moh_show_classes(struct ast_cli_entry *e, int cmd, struc
 }
 
 static struct ast_cli_entry cli_moh[] = {
-	AST_CLI_DEFINE(handle_cli_moh_reload,       "Reload MusicOnHold"),
-	AST_CLI_DEFINE(handle_cli_moh_show_classes, "List MusicOnHold classes"),
-	AST_CLI_DEFINE(handle_cli_moh_show_files,   "List MusicOnHold file-based classes")
+	AST_CLI_DEFINE(handle_cli_moh_reload,       	"Reload MusicOnHold"),
+	AST_CLI_DEFINE(handle_cli_moh_show_classes, 	"List MusicOnHold classes"),
+	AST_CLI_DEFINE(handle_cli_moh_show_files,   	"List MusicOnHold file-based classes"),
+	AST_CLI_DEFINE(handle_cli_moh_unregister_class, "Unregister realtime MusicOnHold class")
 };
 
 static int moh_class_hash(const void *obj, const int flags)
@@ -1977,7 +2114,9 @@ static int load_module(void)
 {
 	int res;
 
-	if (!(mohclasses = ao2_t_container_alloc(53, moh_class_hash, moh_class_cmp, "Moh class container"))) {
+	mohclasses = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 53,
+		moh_class_hash, NULL, moh_class_cmp, "Moh class container");
+	if (!mohclasses) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
