@@ -387,7 +387,9 @@ static void *parking_config_alloc(void)
 		return NULL;
 	}
 
-	if (!(cfg->parking_lots = ao2_container_alloc(37, parking_lot_cfg_hash_fn, parking_lot_cfg_cmp_fn))) {
+	cfg->parking_lots = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+		parking_lot_cfg_hash_fn, NULL, parking_lot_cfg_cmp_fn);
+	if (!cfg->parking_lots) {
 		return NULL;
 	}
 
@@ -724,7 +726,6 @@ static int parking_add_extension(struct ast_context *context, int replace, const
 
 	if (ast_add_extension2_nolock(context, replace, extension, priority, NULL, NULL,
 			application, data_duplicate, ast_free_ptr, registrar)) {
-		ast_free(data_duplicate);
 		return -1;
 	}
 
