@@ -191,6 +191,15 @@ void ast_sip_initialize_global_headers(void);
  */
 void ast_sip_destroy_global_headers(void);
 
+/*!
+ * \internal
+ * \brief Pre-initialize OPTIONS request handling.
+ *
+ * \retval 0 on success
+ * \retval other on failure
+ */
+int ast_res_pjsip_preinit_options_handling(void);
+
 /*!
  * \internal
  * \brief Initialize OPTIONS request handling.
@@ -461,4 +470,24 @@ int ast_sip_initialize_transport_management(void);
  */
 void ast_sip_destroy_transport_management(void);
 
+/*!
+ * \internal
+ * \brief Add online persistent endpoints to the given regcontext
+ *
+ * \param regcontext The context to add endpoints to
+ *
+ * \retval -1 on error, 0 on success
+ */
+int ast_sip_persistent_endpoint_add_to_regcontext(const char *regcontext);
+
+enum ast_sip_taskprocessor_overload_trigger {
+	TASKPROCESSOR_OVERLOAD_TRIGGER_NONE = 0,
+	TASKPROCESSOR_OVERLOAD_TRIGGER_GLOBAL,
+	TASKPROCESSOR_OVERLOAD_TRIGGER_PJSIP_ONLY
+};
+
+enum ast_sip_taskprocessor_overload_trigger ast_sip_get_taskprocessor_overload_trigger(void);
+
+const char *ast_sip_overload_trigger_to_str(enum ast_sip_taskprocessor_overload_trigger trigger);
+
 #endif /* RES_PJSIP_PRIVATE_H_ */
