@@ -42,7 +42,7 @@
 				<description>
 					<para>This module provides alternatives to matching inbound requests to
 					a configured endpoint. At least one of the matching mechanisms
-					must be provided, or the object configuration will be invalid.</para>
+					must be provided, or the object configuration is invalid.</para>
 					<para>The matching mechanisms are provided by the following
 					configuration options:</para>
 					<enumlist>
@@ -60,11 +60,26 @@
 					<synopsis>IP addresses or networks to match against.</synopsis>
 					<description>
 						<para>The value is a comma-delimited list of IP addresses or
-						hostnames.  IP addresses may have a subnet mask appended.  The
-						subnet mask may be written in either CIDR or dotted-decimal
-						notation.  Separate the IP address and subnet mask with a slash
-						('/').
-						</para>
+						hostnames.</para>
+						<para>IP addresses may have a subnet mask appended. The subnet
+						mask may be written in either CIDR or dotted-decimal
+						notation. Separate the IP address and subnet mask with a slash
+						('/'). A source port can also be specified by adding a colon (':')
+						after the address but before the subnet mask, e.g.
+						3.2.1.0:5061/24. To specify a source port for an IPv6 address, the
+						address itself must be enclosed in square brackets
+						('[2001:db8:0::1]:5060')</para>
+						<para>When a hostname is used, the behavior depends on whether
+						<replaceable>srv_lookups</replaceable> is enabled and/or a source
+						port is provided. If <replaceable>srv_lookups</replaceable> is
+						enabled and a source port is not provided, Asterisk will perform
+						an SRV lookup on the provided hostname, adding all of the A and
+						AAAA records that are resolved.</para>
+						<para>If the SRV lookup fails,
+						<replaceable>srv_lookups</replaceable> is disabled, or a source
+						port is specified when the hostname is configured, Asterisk will
+						resolve the hostname and add all A and AAAA records that are
+						resolved.</para>
 					</description>
 				</configOption>
 				<configOption name="srv_lookups" default="yes">
@@ -86,6 +101,13 @@
 						specified with a <literal>:</literal>, as in
 						<literal>match_header = SIPHeader: value</literal>.
 						</para>
+						<para>The specified SIP header value can be a regular
+						expression if the value is of the form
+						/<replaceable>regex</replaceable>/.
+						</para>
+						<note><para>Use of a regex is expensive so be sure you need
+						to use a regex to match your endpoint.
+						</para></note>
 					</description>
 				</configOption>
 				<configOption name="type">
@@ -109,13 +131,21 @@ struct ip_identify_match {
 		AST_STRING_FIELD(endpoint_name);
 		/*! If matching by header, the header/value to match against */
 		AST_STRING_FIELD(match_header);
+		/*! SIP header name of the match_header string */
+		AST_STRING_FIELD(match_header_name);
+		/*! SIP header value of the match_header string */
+		AST_STRING_FIELD(match_header_value);
 	);
+	/*! Compiled match_header regular expression when is_regex is non-zero */
+	regex_t regex_buf;
 	/*! \brief Networks or addresses that should match this */
 	struct ast_ha *matches;
-	/*! \brief Perform SRV resolution of hostnames */
-	unsigned int srv_lookups;
 	/*! \brief Hosts to be resolved when applying configuration */
 	struct ao2_container *hosts;
+	/*! \brief Perform SRV resolution of hostnames */
+	unsigned int srv_lookups;
+	/*! Non-zero if match_header has a regular expression (i.e., regex_buf is valid) */
+	unsigned int is_regex:1;
 };
 
 /*! \brief Destructor function for a matching object */
@@ -126,6 +156,9 @@ static void ip_identify_destroy(void *obj)
 	ast_string_field_free_memory(identify);
 	ast_free_ha(identify->matches);
 	ao2_cleanup(identify->hosts);
+	if (identify->is_regex) {
+		regfree(&identify->regex_buf);
+	}
 }
 
 /*! \brief Allocator function for a matching object */
@@ -146,47 +179,66 @@ static int header_identify_match_check(void *obj, void *arg, int flags)
 {
 	struct ip_identify_match *identify = obj;
 	struct pjsip_rx_data *rdata = arg;
-	pjsip_generic_string_hdr *header;
+	pjsip_hdr *header;
 	pj_str_t pj_header_name;
-	pj_str_t pj_header_value;
-	char *c_header;
-	char *c_value;
+	int header_present;
 
 	if (ast_strlen_zero(identify->match_header)) {
 		return 0;
 	}
 
-	c_header = ast_strdupa(identify->match_header);
-	c_value = strchr(c_header, ':');
-	if (!c_value) {
-		/* This should not be possible.  The object cannot be created if so. */
-		ast_assert(0);
-		return 0;
-	}
-	*c_value = '\0';
-	c_value++;
-	c_value = ast_strip(c_value);
+	pj_header_name = pj_str((void *) identify->match_header_name);
 
-	pj_header_name = pj_str(c_header);
-	header = pjsip_msg_find_hdr_by_name(rdata->msg_info.msg, &pj_header_name, NULL);
-	if (!header) {
-		ast_debug(3, "Identify '%s': SIP message does not have header '%s'\n",
+	/* Check all headers of the given name for a match. */
+	header_present = 0;
+	for (header = NULL;
+		(header = pjsip_msg_find_hdr_by_name(rdata->msg_info.msg, &pj_header_name, header));
+		header = header->next) {
+		char *pos;
+		int len;
+		char buf[PATH_MAX];
+
+		header_present = 1;
+
+		/* Print header line to buf */
+		len = pjsip_hdr_print_on(header, buf, sizeof(buf) - 1);
+		if (len < 0) {
+			/* Buffer not large enough or no header vptr! */
+			ast_assert(0);
+			continue;
+		}
+		buf[len] = '\0';
+
+		/* Remove header name from pj_buf and trim blanks. */
+		pos = strchr(buf, ':');
+		if (!pos) {
+			/* No header name?  Bug in PJPROJECT if so. */
+			ast_assert(0);
+			continue;
+		}
+		pos = ast_strip(pos + 1);
+
+		/* Does header value match what we are looking for? */
+		if (identify->is_regex) {
+			if (!regexec(&identify->regex_buf, pos, 0, NULL, 0)) {
+				return CMP_MATCH;
+			}
+		} else if (!strcmp(identify->match_header_value, pos)) {
+			return CMP_MATCH;
+		}
+
+		ast_debug(3, "Identify '%s': SIP message has '%s' header but value '%s' does not match '%s'.\n",
 			ast_sorcery_object_get_id(identify),
-			c_header);
-		return 0;
+			identify->match_header_name,
+			pos,
+			identify->match_header_value);
 	}
-
-	pj_header_value = pj_str(c_value);
-	if (pj_strcmp(&pj_header_value, &header->hvalue)) {
-		ast_debug(3, "Identify '%s': SIP message has header '%s' but value '%.*s' does not match '%s'\n",
+	if (!header_present) {
+		ast_debug(3, "Identify '%s': SIP message does not have '%s' header.\n",
 			ast_sorcery_object_get_id(identify),
-			c_header,
-			(int) pj_strlen(&header->hvalue), pj_strbuf(&header->hvalue),
-			c_value);
-		return 0;
+			identify->match_header_name);
 	}
-
-	return CMP_MATCH;
+	return 0;
 }
 
 /*! \brief Comparator function for matching an object by IP address */
@@ -273,7 +325,7 @@ static int ip_identify_match_host_lookup(struct ip_identify_match *identify, con
 	int num_addrs = 0, error = 0, i;
 	int results = 0;
 
-	num_addrs = ast_sockaddr_resolve(&addrs, host, PARSE_PORT_FORBID, AST_AF_UNSPEC);
+	num_addrs = ast_sockaddr_resolve(&addrs, host, 0, AST_AF_UNSPEC);
 	if (!num_addrs) {
 		return -1;
 	}
@@ -285,7 +337,7 @@ static int ip_identify_match_host_lookup(struct ip_identify_match *identify, con
 		}
 
 		/* We deny what we actually want to match because there is an implicit permit all rule for ACLs */
-		identify->matches = ast_append_ha("d", ast_sockaddr_stringify_addr(&addrs[i]), identify->matches, &error);
+		identify->matches = ast_append_ha_with_port("d", ast_sockaddr_stringify(&addrs[i]), identify->matches, &error);
 
 		if (!identify->matches || error) {
 			results = -1;
@@ -343,15 +395,20 @@ static int ip_identify_match_handler(const struct aco_option *opt, struct ast_va
 	}
 
 	while ((current_string = ast_strip(strsep(&input_string, ",")))) {
-		char *mask = strrchr(current_string, '/');
+		char *mask;
+		struct ast_sockaddr address;
 		int error = 0;
 
 		if (ast_strlen_zero(current_string)) {
 			continue;
 		}
 
-		if (mask) {
-			identify->matches = ast_append_ha("d", current_string, identify->matches, &error);
+		mask = strrchr(current_string, '/');
+
+		/* If it looks like a netmask is present, or we can immediately parse as an IP,
+		 * hand things off to the ACL */
+		if (mask || ast_sockaddr_parse(&address, current_string, 0)) {
+			identify->matches = ast_append_ha_with_port("d", current_string, identify->matches, &error);
 
 			if (!identify->matches || error) {
 				ast_log(LOG_ERROR, "Failed to add address '%s' to ip endpoint identifier '%s'\n",
@@ -404,34 +461,80 @@ static int ip_identify_apply(const struct ast_sorcery *sorcery, void *obj)
 			ast_sorcery_object_get_id(identify));
 		return -1;
 	}
-	if (!ast_strlen_zero(identify->match_header)
-		&& !strchr(identify->match_header, ':')) {
-		ast_log(LOG_ERROR, "Identify '%s' missing ':' separator in match_header '%s'.\n",
-			ast_sorcery_object_get_id(identify), identify->match_header);
-		return -1;
+
+	if (!ast_strlen_zero(identify->match_header)) {
+		char *c_header;
+		char *c_value;
+		int len;
+
+		/* Split the header name and value */
+		c_header = ast_strdupa(identify->match_header);
+		c_value = strchr(c_header, ':');
+		if (!c_value) {
+			ast_log(LOG_ERROR, "Identify '%s' missing ':' separator in match_header '%s'.\n",
+				ast_sorcery_object_get_id(identify), identify->match_header);
+			return -1;
+		}
+		*c_value = '\0';
+		c_value = ast_strip(c_value + 1);
+		c_header = ast_strip(c_header);
+
+		if (ast_strlen_zero(c_header)) {
+			ast_log(LOG_ERROR, "Identify '%s' has no SIP header to match in match_header '%s'.\n",
+				ast_sorcery_object_get_id(identify), identify->match_header);
+			return -1;
+		}
+
+		if (!strcmp(c_value, "//")) {
+			/* An empty regex is the same as an empty literal string. */
+			c_value = "";
+		}
+
+		if (ast_string_field_set(identify, match_header_name, c_header)
+			|| ast_string_field_set(identify, match_header_value, c_value)) {
+			return -1;
+		}
+
+		len = strlen(c_value);
+		if (2 < len && c_value[0] == '/' && c_value[len - 1] == '/') {
+			/* Make "/regex/" into "regex" */
+			c_value[len - 1] = '\0';
+			++c_value;
+
+			if (regcomp(&identify->regex_buf, c_value, REG_EXTENDED | REG_NOSUB)) {
+				ast_log(LOG_ERROR, "Identify '%s' failed to compile match_header regex '%s'.\n",
+					ast_sorcery_object_get_id(identify), c_value);
+				return -1;
+			}
+			identify->is_regex = 1;
+		}
 	}
 
 	if (!identify->hosts) {
+		/* No match addresses to resolve */
 		return 0;
 	}
 
 	/* Resolve the match addresses now */
 	i = ao2_iterator_init(identify->hosts, 0);
 	while ((current_string = ao2_iterator_next(&i))) {
-		struct ast_sockaddr address;
 		int results = 0;
+		char *colon = strrchr(current_string, ':');
 
-		/* If the provided string is not an IP address perform SRV resolution on it */
-		if (identify->srv_lookups && !ast_sockaddr_parse(&address, current_string, 0)) {
-			results = ip_identify_match_srv_lookup(identify, "_sip._udp", current_string,
-				results);
-			if (results != -1) {
-				results = ip_identify_match_srv_lookup(identify, "_sip._tcp",
-					current_string, results);
-			}
-			if (results != -1) {
-				results = ip_identify_match_srv_lookup(identify, "_sips._tcp",
-					current_string, results);
+		/* We skip SRV lookup if a colon is present, assuming a port was specified */
+		if (!colon) {
+			/* No port, and we know this is not an IP address, so perform SRV resolution on it */
+			if (identify->srv_lookups) {
+				results = ip_identify_match_srv_lookup(identify, "_sip._udp", current_string,
+					results);
+				if (results != -1) {
+					results = ip_identify_match_srv_lookup(identify, "_sip._tcp",
+						current_string, results);
+				}
+				if (results != -1) {
+					results = ip_identify_match_srv_lookup(identify, "_sips._tcp",
+						current_string, results);
+				}
 			}
 		}
 
@@ -471,20 +574,31 @@ static int match_to_str(const void *obj, const intptr_t *args, char **buf)
 	return 0;
 }
 
-static int match_to_var_list(const void *obj, struct ast_variable **fields)
+static void match_to_var_list_append(struct ast_variable **head, struct ast_ha *ha)
 {
 	char str[MAX_OBJECT_FIELD];
+	const char *addr;
+
+	if (ast_sockaddr_port(&ha->addr)) {
+		addr = ast_strdupa(ast_sockaddr_stringify(&ha->addr));
+	} else {
+		addr = ast_strdupa(ast_sockaddr_stringify_addr(&ha->addr));
+	}
+
+	snprintf(str, MAX_OBJECT_FIELD, "%s%s/%s", ha->sense == AST_SENSE_ALLOW ? "!" : "",
+			 addr, ast_sockaddr_stringify_addr(&ha->netmask));
+
+	ast_variable_list_append(head, ast_variable_new("match", str, ""));
+}
+
+static int match_to_var_list(const void *obj, struct ast_variable **fields)
+{
 	const struct ip_identify_match *identify = obj;
 	struct ast_variable *head = NULL;
 	struct ast_ha *ha = identify->matches;
 
 	for (; ha; ha = ha->next) {
-		const char *addr = ast_strdupa(ast_sockaddr_stringify_addr(&ha->addr));
-		snprintf(str, MAX_OBJECT_FIELD, "%s%s/%s", ha->sense == AST_SENSE_ALLOW ? "!" : "",
-			addr, ast_sockaddr_stringify_addr(&ha->netmask));
-
-		ast_variable_list_append(&head, ast_variable_new("match", str, ""));
-
+		match_to_var_list_append(&head, ha);
 	}
 
 	if (head) {
@@ -653,7 +767,13 @@ static int cli_print_body(void *obj, void *arg, int flags)
 		indent = CLI_INDENT_TO_SPACES(context->indent_level);
 
 		for (match = ident->matches; match; match = match->next) {
-			const char *addr = ast_sockaddr_stringify_addr(&match->addr);
+			const char *addr;
+
+			if (ast_sockaddr_port(&match->addr)) {
+				addr = ast_sockaddr_stringify(&match->addr);
+			} else {
+				addr = ast_sockaddr_stringify_addr(&match->addr);
+			}
 
 			ast_str_append(&context->output_buffer, 0, "%*s: %s%s/%d\n",
 				indent,
@@ -768,6 +888,7 @@ static int unload_module(void)
 	ast_cli_unregister_multiple(cli_identify, ARRAY_LEN(cli_identify));
 	ast_sip_unregister_cli_formatter(cli_formatter);
 	ast_sip_unregister_endpoint_formatter(&endpoint_identify_formatter);
+	ast_sip_unregister_endpoint_identifier(&header_identifier);
 	ast_sip_unregister_endpoint_identifier(&ip_identifier);
 
 	return 0;
