@@ -81,12 +81,12 @@ static int pj_max_hostname = PJ_MAX_HOSTNAME;
 static int pjsip_max_url_size = PJSIP_MAX_URL_SIZE;
 
 /*! \brief Internal function which returns the expiration time for a contact */
-static int registrar_get_expiration(const struct ast_sip_aor *aor, const pjsip_contact_hdr *contact, const pjsip_rx_data *rdata)
+static unsigned int registrar_get_expiration(const struct ast_sip_aor *aor, const pjsip_contact_hdr *contact, const pjsip_rx_data *rdata)
 {
 	pjsip_expires_hdr *expires;
-	int expiration = aor->default_expiration;
+	unsigned int expiration = aor->default_expiration;
 
-	if (contact && contact->expires != -1) {
+	if (contact && contact->expires != PJSIP_EXPIRES_NOT_SPECIFIED) {
 		/* Expiration was provided with the contact itself */
 		expiration = contact->expires;
 	} else if ((expires = pjsip_msg_find_hdr(rdata->msg_info.msg, PJSIP_H_EXPIRES, NULL))) {
@@ -122,42 +122,47 @@ static int registrar_find_contact(void *obj, void *arg, int flags)
 {
 	struct ast_sip_contact *contact = obj;
 	const struct registrar_contact_details *details = arg;
-	pjsip_uri *contact_uri = pjsip_parse_uri(details->pool, (char*)contact->uri, strlen(contact->uri), 0);
+	pjsip_uri *contact_uri;
+
+	if (ast_tvzero(contact->expiration_time)) {
+		return 0;
+	}
+
+	contact_uri = pjsip_parse_uri(details->pool, (char*)contact->uri, strlen(contact->uri), 0);
+	if (!contact_uri) {
+		ast_log(LOG_WARNING, "Unable to parse contact URI from '%s'.\n", contact->uri);
+		return 0;
+	}
 
 	return (pjsip_uri_cmp(PJSIP_URI_IN_CONTACT_HDR, details->uri, contact_uri) == PJ_SUCCESS) ? CMP_MATCH : 0;
 }
 
 /*! \brief Internal function which validates provided Contact headers to confirm that they are acceptable, and returns number of contacts */
-static int registrar_validate_contacts(const pjsip_rx_data *rdata, struct ao2_container *contacts, struct ast_sip_aor *aor, int *added, int *updated, int *deleted)
+static int registrar_validate_contacts(const pjsip_rx_data *rdata, pj_pool_t *pool, struct ao2_container *contacts,
+	struct ast_sip_aor *aor, int permanent, int *added, int *updated, int *deleted)
 {
 	pjsip_contact_hdr *previous = NULL;
 	pjsip_contact_hdr *contact = (pjsip_contact_hdr *)&rdata->msg_info.msg->hdr;
 	struct registrar_contact_details details = {
-		.pool = pjsip_endpt_create_pool(ast_sip_get_pjsip_endpoint(), "Contact Comparison", 256, 256),
+		.pool = pool,
 	};
 
-	if (!details.pool) {
-		return -1;
-	}
-
-	while ((contact = (pjsip_contact_hdr *) pjsip_msg_find_hdr(rdata->msg_info.msg, PJSIP_H_CONTACT, contact->next))) {
-		int expiration = registrar_get_expiration(aor, contact, rdata);
+	for (; (contact = (pjsip_contact_hdr *) pjsip_msg_find_hdr(rdata->msg_info.msg, PJSIP_H_CONTACT, contact->next)); pj_pool_reset(pool)) {
+		unsigned int expiration = registrar_get_expiration(aor, contact, rdata);
 		struct ast_sip_contact *existing;
 		char contact_uri[pjsip_max_url_size];
 
 		if (contact->star) {
 			/* The expiration MUST be 0 when a '*' contact is used and there must be no other contact */
 			if (expiration != 0 || previous) {
-				pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
 				return -1;
 			}
 			/* Count all contacts to delete */
-			*deleted = ao2_container_count(contacts);
+			*deleted = ao2_container_count(contacts) - permanent;
 			previous = contact;
 			continue;
 		} else if (previous && previous->star) {
 			/* If there is a previous contact and it is a '*' this is a deal breaker */
-			pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
 			return -1;
 		}
 		previous = contact;
@@ -171,13 +176,11 @@ static int registrar_validate_contacts(const pjsip_rx_data *rdata, struct ao2_co
 		/* pjsip_uri_print returns -1 if there's not enough room in the buffer */
 		if (pjsip_uri_print(PJSIP_URI_IN_CONTACT_HDR, details.uri, contact_uri, sizeof(contact_uri)) < 0) {
 			/* If the total length of the uri is greater than pjproject can handle, go no further */
-			pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
 			return -1;
 		}
 
 		if (details.uri->host.slen >= pj_max_hostname) {
 			/* If the length of the hostname is greater than pjproject can handle, go no further */
-			pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
 			return -1;
 		}
 
@@ -195,38 +198,25 @@ static int registrar_validate_contacts(const pjsip_rx_data *rdata, struct ao2_co
 		}
 	}
 
-	/* The provided contacts are acceptable, huzzah! */
-	pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
 	return 0;
 }
 
-/*! \brief Callback function which prunes static contacts */
-static int registrar_prune_static(void *obj, void *arg, int flags)
-{
-	struct ast_sip_contact *contact = obj;
+enum contact_delete_type {
+	CONTACT_DELETE_ERROR,
+	CONTACT_DELETE_EXISTING,
+	CONTACT_DELETE_EXPIRE,
+	CONTACT_DELETE_REQUEST,
+	CONTACT_DELETE_SHUTDOWN,
+};
 
-	return ast_tvzero(contact->expiration_time) ? CMP_MATCH : 0;
-}
+static int registrar_contact_delete(enum contact_delete_type type, pjsip_transport *transport,
+	struct ast_sip_contact *contact, const char *aor_name);
 
 /*! \brief Internal function used to delete a contact from an AOR */
 static int registrar_delete_contact(void *obj, void *arg, int flags)
 {
-	struct ast_sip_contact *contact = obj;
-	const char *aor_name = arg;
-
-	ast_sip_location_delete_contact(contact);
-	if (!ast_strlen_zero(aor_name)) {
-		ast_verb(3, "Removed contact '%s' from AOR '%s' due to request\n", contact->uri, aor_name);
-		ast_test_suite_event_notify("AOR_CONTACT_REMOVED",
-				"Contact: %s\r\n"
-				"AOR: %s\r\n"
-				"UserAgent: %s",
-				contact->uri,
-				aor_name,
-				contact->user_agent);
-	}
-
-	return CMP_MATCH;
+	return registrar_contact_delete(
+		CONTACT_DELETE_REQUEST, NULL, obj, arg) ? 0 : CMP_MATCH;
 }
 
 /*! \brief Internal function which adds a contact to a response */
@@ -234,14 +224,25 @@ static int registrar_add_contact(void *obj, void *arg, int flags)
 {
 	struct ast_sip_contact *contact = obj;
 	pjsip_tx_data *tdata = arg;
-	pjsip_contact_hdr *hdr = pjsip_contact_hdr_create(tdata->pool);
 	pj_str_t uri;
+	pjsip_uri *parsed;
 
 	pj_strdup2_with_null(tdata->pool, &uri, contact->uri);
-	hdr->uri = pjsip_parse_uri(tdata->pool, uri.ptr, uri.slen, PJSIP_PARSE_URI_AS_NAMEADDR);
-	hdr->expires = ast_tvdiff_ms(contact->expiration_time, ast_tvnow()) / 1000;
+	parsed = pjsip_parse_uri(tdata->pool, uri.ptr, uri.slen, PJSIP_PARSE_URI_AS_NAMEADDR);
 
-	pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr*)hdr);
+	if (parsed && (PJSIP_URI_SCHEME_IS_SIP(parsed) || PJSIP_URI_SCHEME_IS_SIPS(parsed))) {
+		pjsip_contact_hdr *hdr = pjsip_contact_hdr_create(tdata->pool);
+		hdr->uri = parsed;
+		if (!ast_tvzero(contact->expiration_time)) {
+			hdr->expires = ast_tvdiff_ms(contact->expiration_time, ast_tvnow()) / 1000;
+		} else {
+			hdr->expires = PJSIP_EXPIRES_NOT_SPECIFIED;
+		}
+		pjsip_msg_add_hdr(tdata->msg, (pjsip_hdr *) hdr);
+	} else {
+		ast_log(LOG_WARNING, "Skipping invalid Contact URI \"%.*s\" for AOR %s\n",
+			(int) uri.slen, uri.ptr, contact->aor);
+	}
 
 	return 0;
 }
@@ -270,7 +271,7 @@ static int build_path_data(pjsip_rx_data *rdata, struct ast_str **path_str)
 	}
 
 	*path_str = ast_str_create(64);
-	if (!path_str) {
+	if (!*path_str) {
 		return -1;
 	}
 
@@ -324,6 +325,8 @@ struct contact_transport_monitor {
 	 * \note Stored after aor_name in space reserved when struct allocated.
 	 */
 	char *contact_name;
+	/*! Indicates that the monitor is in the process of removing a contact */
+	int removing;
 	/*! AOR name the contact is associated */
 	char aor_name[0];
 };
@@ -337,7 +340,7 @@ static int contact_transport_monitor_matcher(void *a, void *b)
 		&& strcmp(ma->contact_name, mb->contact_name) == 0;
 }
 
-static void register_contact_transport_shutdown_cb(void *data)
+static int register_contact_transport_remove_cb(void *data)
 {
 	struct contact_transport_monitor *monitor = data;
 	struct ast_sip_contact *contact;
@@ -345,26 +348,145 @@ static void register_contact_transport_shutdown_cb(void *data)
 
 	lock = ast_named_lock_get(AST_NAMED_LOCK_TYPE_MUTEX, "aor", monitor->aor_name);
 	if (!lock) {
-		return;
+		ao2_lock(monitor);
+		monitor->removing = 0;
+		ao2_unlock(monitor);
+		ao2_ref(monitor, -1);
+		return 0;
 	}
 
 	ao2_lock(lock);
+
 	contact = ast_sip_location_retrieve_contact(monitor->contact_name);
 	if (contact) {
-		ast_sip_location_delete_contact(contact);
-		ast_verb(3, "Removed contact '%s' from AOR '%s' due to transport shutdown\n",
-			contact->uri, monitor->aor_name);
-		ast_test_suite_event_notify("AOR_CONTACT_REMOVED",
-			"Contact: %s\r\n"
-			"AOR: %s\r\n"
-			"UserAgent: %s",
-			contact->uri,
-			monitor->aor_name,
-			contact->user_agent);
+		registrar_contact_delete(CONTACT_DELETE_SHUTDOWN, NULL, contact, monitor->aor_name);
 		ao2_ref(contact, -1);
 	}
+
 	ao2_unlock(lock);
 	ast_named_lock_put(lock);
+
+	ao2_ref(monitor, -1);
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief The reliable transport we registered as a contact has shutdown.
+ *
+ * \param data What contact needs to be removed.
+ *
+ * \note Normally executed by the pjsip monitor thread.
+ *
+ * \return Nothing
+ */
+static void register_contact_transport_shutdown_cb(void *data)
+{
+	struct contact_transport_monitor *monitor = data;
+
+	/*
+	 * It's possible for this shutdown handler to get called multiple times for the
+	 * same monitor from different threads. Only one of the calls needs to do the
+	 * actual removing of the contact, so if one is currently removing then any
+	 * subsequent calls can skip.
+	 */
+	ao2_lock(monitor);
+	if (monitor->removing) {
+		ao2_unlock(monitor);
+		return;
+	}
+
+	monitor->removing = 1;
+
+	/*
+	 * Push off to a default serializer.  This is in case sorcery
+	 * does database accesses for contacts.  Database accesses may
+	 * not be on this machine.  We don't want to tie up the pjsip
+	 * monitor thread with potentially long access times.
+	 */
+	ao2_ref(monitor, +1);
+	if (ast_sip_push_task(NULL, register_contact_transport_remove_cb, monitor)) {
+		monitor->removing = 0;
+		ao2_ref(monitor, -1);
+	}
+
+	ao2_unlock(monitor);
+}
+
+
+static int registrar_contact_delete(enum contact_delete_type type, pjsip_transport *transport,
+	struct ast_sip_contact *contact, const char *aor_name)
+{
+	int aor_size;
+
+	/* Permanent contacts can't be deleted */
+	if (ast_tvzero(contact->expiration_time)) {
+		return -1;
+	}
+
+	aor_size = aor_name ? strlen(aor_name) : 0;
+	if (contact->prune_on_boot && type != CONTACT_DELETE_SHUTDOWN && aor_size) {
+		const char *contact_name = ast_sorcery_object_get_id(contact);
+		size_t contact_name_len = strlen(contact_name) + 1;
+		struct contact_transport_monitor *monitor = ast_alloca(
+			sizeof(*monitor) + 1 + aor_size + contact_name_len);
+
+		strcpy(monitor->aor_name, aor_name); /* Safe */
+		monitor->contact_name = monitor->aor_name + aor_size + 1;
+		ast_copy_string(monitor->contact_name, contact_name, contact_name_len); /* Safe */
+
+		if (transport) {
+			ast_sip_transport_monitor_unregister(transport,
+				register_contact_transport_shutdown_cb,	monitor,
+				contact_transport_monitor_matcher);
+		} else {
+			/*
+			 * If a specific transport is not supplied then unregister the matching
+			 * monitor from all reliable transports.
+			 */
+			ast_sip_transport_monitor_unregister_all(register_contact_transport_shutdown_cb,
+				monitor, contact_transport_monitor_matcher);
+		}
+	}
+
+	ast_sip_location_delete_contact(contact);
+
+	if (aor_size) {
+		if (VERBOSITY_ATLEAST(3)) {
+			const char *reason = "none";
+
+			switch (type) {
+			case CONTACT_DELETE_ERROR:
+				reason = "registration failure";
+				break;
+			case CONTACT_DELETE_EXISTING:
+				reason = "remove existing";
+				break;
+			case CONTACT_DELETE_EXPIRE:
+				reason = "expiration";
+				break;
+			case CONTACT_DELETE_REQUEST:
+				reason = "request";
+				break;
+			case CONTACT_DELETE_SHUTDOWN:
+				reason = "shutdown";
+				break;
+			}
+
+			ast_verb(3, "Removed contact '%s' from AOR '%s' due to %s\n",
+					 contact->uri, aor_name, reason);
+		}
+
+		ast_test_suite_event_notify("AOR_CONTACT_REMOVED",
+				"Contact: %s\r\n"
+				"AOR: %s\r\n"
+				"UserAgent: %s",
+				contact->uri,
+				aor_name,
+				contact->user_agent);
+	}
+
+	return 0;
 }
 
 AST_VECTOR(excess_contact_vector, struct ast_sip_contact *);
@@ -412,7 +534,8 @@ static int vec_contact_add(void *obj, void *arg, int flags)
  *
  * \return Nothing
  */
-static void remove_excess_contacts(struct ao2_container *contacts, unsigned int to_remove)
+static void remove_excess_contacts(struct ao2_container *contacts, struct ao2_container *response_contacts,
+	unsigned int to_remove)
 {
 	struct excess_contact_vector contact_vec;
 
@@ -442,21 +565,29 @@ static void remove_excess_contacts(struct ao2_container *contacts, unsigned int
 
 		contact = AST_VECTOR_GET(&contact_vec, to_remove);
 
-		ast_sip_location_delete_contact(contact);
-		ast_verb(3, "Removed contact '%s' from AOR '%s' due to remove_existing\n",
-			contact->uri, contact->aor);
-		ast_test_suite_event_notify("AOR_CONTACT_REMOVED",
-			"Contact: %s\r\n"
-			"AOR: %s\r\n"
-			"UserAgent: %s",
-			contact->uri,
-			contact->aor,
-			contact->user_agent);
+		registrar_contact_delete(CONTACT_DELETE_EXISTING, NULL, contact, contact->aor);
+
+		ao2_unlink(response_contacts, contact);
 	}
 
 	AST_VECTOR_FREE(&contact_vec);
 }
 
+/*! \brief Callback function which adds non-permanent contacts to a container */
+static int registrar_add_non_permanent(void *obj, void *arg, int flags)
+{
+	struct ast_sip_contact *contact = obj;
+	struct ao2_container *container = arg;
+
+	if (ast_tvzero(contact->expiration_time)) {
+		return 0;
+	}
+
+	ao2_link(container, contact);
+
+	return 0;
+}
+
 struct aor_core_response {
 	/*! Tx data to use for statefull response.  NULL for stateless response. */
 	pjsip_tx_data *tdata;
@@ -476,8 +607,10 @@ static void register_aor_core(pjsip_rx_data *rdata,
 	int added = 0;
 	int updated = 0;
 	int deleted = 0;
+	int permanent = 0;
 	int contact_count;
-	pjsip_contact_hdr *contact_hdr = NULL;
+	struct ao2_container *existing_contacts = NULL;
+	pjsip_contact_hdr *contact_hdr = (pjsip_contact_hdr *)&rdata->msg_info.msg->hdr;
 	struct registrar_contact_details details = { 0, };
 	pjsip_tx_data *tdata;
 	RAII_VAR(struct ast_str *, path_str, NULL, ast_free);
@@ -493,15 +626,28 @@ static void register_aor_core(pjsip_rx_data *rdata,
 	char *call_id = NULL;
 	size_t alloc_size;
 
-	/* So we don't count static contacts against max_contacts we prune them out from the container */
-	ao2_callback(contacts, OBJ_NODATA | OBJ_UNLINK | OBJ_MULTIPLE, registrar_prune_static, NULL);
+	/* We create a single pool and use it throughout this function where we need one */
+	details.pool = pjsip_endpt_create_pool(ast_sip_get_pjsip_endpoint(),
+		"Contact Comparison", 1024, 256);
+	if (!details.pool) {
+		response->code = 500;
+		return;
+	}
 
-	if (registrar_validate_contacts(rdata, contacts, aor, &added, &updated, &deleted)) {
+	/* If there are any permanent contacts configured on the AOR we need to take them
+	 * into account when counting contacts.
+	 */
+	if (aor->permanent_contacts) {
+		permanent = ao2_container_count(aor->permanent_contacts);
+	}
+
+	if (registrar_validate_contacts(rdata, details.pool, contacts, aor, permanent, &added, &updated, &deleted)) {
 		/* The provided Contact headers do not conform to the specification */
 		ast_sip_report_failed_acl(endpoint, rdata, "registrar_invalid_contacts_provided");
 		ast_log(LOG_WARNING, "Failed to validate contacts in REGISTER request from '%s'\n",
 				ast_sorcery_object_get_id(endpoint));
 		response->code = 400;
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
 		return;
 	}
 
@@ -510,15 +656,29 @@ static void register_aor_core(pjsip_rx_data *rdata,
 		ast_log(LOG_WARNING, "Invalid modifications made to REGISTER request from '%s' by intervening proxy\n",
 				ast_sorcery_object_get_id(endpoint));
 		response->code = 420;
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
 		return;
 	}
 
 	if (aor->remove_existing) {
 		/* Cumulative number of contacts affected by this registration */
 		contact_count = MAX(updated + added - deleted,  0);
+
+		/* We need to keep track of only existing contacts so we can later
+		 * remove them if need be.
+		 */
+		existing_contacts = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0,
+			NULL, ast_sorcery_object_id_compare);
+		if (!existing_contacts) {
+			response->code = 500;
+			pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
+			return;
+		}
+
+		ao2_callback(contacts, OBJ_NODATA, registrar_add_non_permanent, existing_contacts);
 	} else {
 		/* Total contacts after this registration */
-		contact_count = ao2_container_count(contacts) + added - deleted;
+		contact_count = ao2_container_count(contacts) - permanent + added - deleted;
 	}
 	if (contact_count > aor->max_contacts) {
 		/* Enforce the maximum number of contacts */
@@ -526,13 +686,8 @@ static void register_aor_core(pjsip_rx_data *rdata,
 		ast_log(LOG_WARNING, "Registration attempt from endpoint '%s' to AOR '%s' will exceed max contacts of %u\n",
 				ast_sorcery_object_get_id(endpoint), aor_name, aor->max_contacts);
 		response->code = 403;
-		return;
-	}
-
-	details.pool = pjsip_endpt_create_pool(ast_sip_get_pjsip_endpoint(),
-		"Contact Comparison", 256, 256);
-	if (!details.pool) {
-		response->code = 500;
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), details.pool);
+		ao2_cleanup(existing_contacts);
 		return;
 	}
 
@@ -565,7 +720,7 @@ static void register_aor_core(pjsip_rx_data *rdata,
 	}
 
 	/* Iterate each provided Contact header and add, update, or delete */
-	while ((contact_hdr = pjsip_msg_find_hdr(rdata->msg_info.msg, PJSIP_H_CONTACT, contact_hdr ? contact_hdr->next : NULL))) {
+	for (; (contact_hdr = (pjsip_contact_hdr *) pjsip_msg_find_hdr(rdata->msg_info.msg, PJSIP_H_CONTACT, contact_hdr->next)); pj_pool_reset(details.pool)) {
 		int expiration;
 		char contact_uri[pjsip_max_url_size];
 		RAII_VAR(struct ast_sip_contact *, contact, NULL, ao2_cleanup);
@@ -574,6 +729,13 @@ static void register_aor_core(pjsip_rx_data *rdata,
 			/* A star means to unregister everything, so do so for the possible contacts */
 			ao2_callback(contacts, OBJ_NODATA | OBJ_UNLINK | OBJ_MULTIPLE,
 				registrar_delete_contact, (void *)aor_name);
+			/* If we are keeping track of existing contacts for removal then, well, there is
+			 * absolutely nothing left so no need to try to remove any.
+			 */
+			if (existing_contacts) {
+				ao2_ref(existing_contacts, -1);
+				existing_contacts = NULL;
+			}
 			break;
 		}
 
@@ -587,6 +749,14 @@ static void register_aor_core(pjsip_rx_data *rdata,
 		pjsip_uri_print(PJSIP_URI_IN_CONTACT_HDR, details.uri, contact_uri, sizeof(contact_uri));
 
 		contact = ao2_callback(contacts, OBJ_UNLINK, registrar_find_contact, &details);
+
+		/* If a contact was returned and we need to keep track of existing contacts then it
+		 * should be removed.
+		 */
+		if (contact && existing_contacts) {
+			ao2_unlink(existing_contacts, contact);
+		}
+
 		if (!contact) {
 			int prune_on_boot;
 
@@ -610,6 +780,7 @@ static void register_aor_core(pjsip_rx_data *rdata,
 			}
 
 			if (prune_on_boot) {
+				size_t contact_name_len;
 				const char *contact_name;
 				struct contact_transport_monitor *monitor;
 
@@ -618,15 +789,16 @@ static void register_aor_core(pjsip_rx_data *rdata,
 				 * the contact won't be valid anymore if that happens.
 				 */
 				contact_name = ast_sorcery_object_get_id(contact);
-				monitor = ao2_alloc_options(sizeof(*monitor) + 2 + strlen(aor_name)
-					+ strlen(contact_name), NULL, AO2_ALLOC_OPT_LOCK_NOLOCK);
+				contact_name_len = strlen(contact_name) + 1;
+				monitor = ao2_alloc(sizeof(*monitor) + 1 + strlen(aor_name)
+					+ contact_name_len, NULL);
 				if (monitor) {
 					strcpy(monitor->aor_name, aor_name);/* Safe */
 					monitor->contact_name = monitor->aor_name + strlen(aor_name) + 1;
-					strcpy(monitor->contact_name, contact_name);/* Safe */
+					ast_copy_string(monitor->contact_name, contact_name, contact_name_len);/* Safe */
 
-					ast_sip_transport_monitor_register(rdata->tp_info.transport,
-						register_contact_transport_shutdown_cb, monitor);
+					ast_sip_transport_monitor_register_replace(rdata->tp_info.transport,
+						register_contact_transport_shutdown_cb, monitor, contact_transport_monitor_matcher);
 					ao2_ref(monitor, -1);
 				}
 			}
@@ -642,6 +814,8 @@ static void register_aor_core(pjsip_rx_data *rdata,
 					aor_name,
 					expiration,
 					user_agent);
+
+			ao2_link(contacts, contact);
 		} else if (expiration) {
 			struct ast_sip_contact *contact_update;
 
@@ -668,7 +842,8 @@ static void register_aor_core(pjsip_rx_data *rdata,
 			if (ast_sip_location_update_contact(contact_update)) {
 				ast_log(LOG_ERROR, "Failed to update contact '%s' expiration time to %d seconds.\n",
 					contact->uri, expiration);
-				ast_sip_location_delete_contact(contact);
+				registrar_contact_delete(CONTACT_DELETE_ERROR, rdata->tp_info.transport,
+					contact, aor_name);
 				continue;
 			}
 			ast_debug(3, "Refreshed contact '%s' on AOR '%s' with new expiration of %d seconds\n",
@@ -682,33 +857,11 @@ static void register_aor_core(pjsip_rx_data *rdata,
 					aor_name,
 					expiration,
 					contact_update->user_agent);
+			ao2_link(contacts, contact_update);
 			ao2_cleanup(contact_update);
 		} else {
-			if (contact->prune_on_boot) {
-				struct contact_transport_monitor *monitor;
-				const char *contact_name =
-					ast_sorcery_object_get_id(contact);
-
-				monitor = ast_alloca(sizeof(*monitor) + 2 + strlen(aor_name)
-					+ strlen(contact_name));
-				strcpy(monitor->aor_name, aor_name);/* Safe */
-				monitor->contact_name = monitor->aor_name + strlen(aor_name) + 1;
-				strcpy(monitor->contact_name, contact_name);/* Safe */
-
-				ast_sip_transport_monitor_unregister(rdata->tp_info.transport,
-					register_contact_transport_shutdown_cb, monitor, contact_transport_monitor_matcher);
-			}
-
-			/* We want to report the user agent that was actually in the removed contact */
-			ast_sip_location_delete_contact(contact);
-			ast_verb(3, "Removed contact '%s' from AOR '%s' due to request\n", contact_uri, aor_name);
-			ast_test_suite_event_notify("AOR_CONTACT_REMOVED",
-					"Contact: %s\r\n"
-					"AOR: %s\r\n"
-					"UserAgent: %s",
-					contact_uri,
-					aor_name,
-					contact->user_agent);
+			registrar_contact_delete(CONTACT_DELETE_REQUEST, rdata->tp_info.transport,
+					contact, aor_name);
 		}
 	}
 
@@ -719,24 +872,20 @@ static void register_aor_core(pjsip_rx_data *rdata,
 	 * that have not been updated/added/deleted as a result of this
 	 * REGISTER do so.
 	 *
-	 * The contacts container currently holds the existing contacts that
-	 * were not affected by this REGISTER.
+	 * The existing contacts container holds all contacts that were not
+	 * involved in this REGISTER.
+	 * The contacts container holds the current contacts of the AOR.
 	 */
-	if (aor->remove_existing) {
+	if (aor->remove_existing && existing_contacts) {
 		/* Total contacts after this registration */
-		contact_count = ao2_container_count(contacts) + updated + added;
+		contact_count = ao2_container_count(existing_contacts) + updated + added;
 		if (contact_count > aor->max_contacts) {
 			/* Remove excess existing contacts that expire the soonest */
-			remove_excess_contacts(contacts, contact_count - aor->max_contacts);
+			remove_excess_contacts(existing_contacts, contacts, contact_count - aor->max_contacts);
 		}
+		ao2_ref(existing_contacts, -1);
 	}
 
-	/* Re-retrieve contacts.  Caller will clean up the original container. */
-	contacts = ast_sip_location_retrieve_aor_contacts_nolock(aor);
-	if (!contacts) {
-		response->code = 500;
-		return;
-	}
 	response_contact = ao2_callback(contacts, 0, NULL, NULL);
 
 	/* Send a response containing all of the contacts (including static) that are present on this AOR */
@@ -752,7 +901,6 @@ static void register_aor_core(pjsip_rx_data *rdata,
 	registrar_add_date_header(tdata);
 
 	ao2_callback(contacts, 0, registrar_add_contact, tdata);
-	ao2_cleanup(contacts);
 
 	if ((expires_hdr = pjsip_msg_find_hdr(rdata->msg_info.msg, PJSIP_H_EXPIRES, NULL))) {
 		expires_hdr = pjsip_expires_hdr_create(tdata->pool, registrar_get_expiration(aor, NULL, rdata));
@@ -819,14 +967,21 @@ static int match_aor(const char *aor_name, const char *id)
 	return 0;
 }
 
-static char *find_aor_name(const char *username, const char *domain, const char *aors)
+static char *find_aor_name(const pj_str_t *pj_username, const pj_str_t *pj_domain, const char *aors)
 {
 	char *configured_aors;
 	char *aors_buf;
 	char *aor_name;
 	char *id_domain;
+	char *username, *domain;
 	struct ast_sip_domain_alias *alias;
 
+	/* Turn these into C style strings for convenience */
+	username = ast_alloca(pj_strlen(pj_username) + 1);
+	ast_copy_pj_str(username, pj_username, pj_strlen(pj_username) + 1);
+	domain = ast_alloca(pj_strlen(pj_domain) + 1);
+	ast_copy_pj_str(domain, pj_domain, pj_strlen(pj_domain) + 1);
+
 	id_domain = ast_alloca(strlen(username) + strlen(domain) + 2);
 	sprintf(id_domain, "%s@%s", username, domain);
 
@@ -845,7 +1000,7 @@ static char *find_aor_name(const char *username, const char *domain, const char
 	if (alias) {
 		char *id_domain_alias = ast_alloca(strlen(username) + strlen(alias->domain) + 2);
 
-		sprintf(id_domain, "%s@%s", username, alias->domain);
+		sprintf(id_domain_alias, "%s@%s", username, alias->domain);
 		ao2_cleanup(alias);
 
 		configured_aors = strcpy(aors_buf, aors);/* Safe */
@@ -876,11 +1031,10 @@ static struct ast_sip_aor *find_registrar_aor(struct pjsip_rx_data *rdata, struc
 {
 	struct ast_sip_aor *aor = NULL;
 	char *aor_name = NULL;
-	char *domain_name;
-	char *username = NULL;
 	int i;
 
 	for (i = 0; i < AST_VECTOR_SIZE(&endpoint->ident_method_order); ++i) {
+		pj_str_t username;
 		pjsip_sip_uri *uri;
 		pjsip_authorization_hdr *header = NULL;
 
@@ -888,18 +1042,22 @@ static struct ast_sip_aor *find_registrar_aor(struct pjsip_rx_data *rdata, struc
 		case AST_SIP_ENDPOINT_IDENTIFY_BY_USERNAME:
 			uri = pjsip_uri_get_uri(rdata->msg_info.to->uri);
 
-			domain_name = ast_alloca(uri->host.slen + 1);
-			ast_copy_pj_str(domain_name, &uri->host, uri->host.slen + 1);
-			username = ast_alloca(uri->user.slen + 1);
-			ast_copy_pj_str(username, &uri->user, uri->user.slen + 1);
+			pj_strassign(&username, &uri->user);
 
 			/*
 			 * We may want to match without any user options getting
 			 * in the way.
+			 *
+			 * Logic adapted from AST_SIP_USER_OPTIONS_TRUNCATE_CHECK for pj_str_t.
 			 */
-			AST_SIP_USER_OPTIONS_TRUNCATE_CHECK(username);
+			if (ast_sip_get_ignore_uri_user_options()) {
+				pj_ssize_t semi = pj_strcspn2(&username, ";");
+				if (semi < pj_strlen(&username)) {
+					username.slen = semi;
+				}
+			}
 
-			aor_name = find_aor_name(username, domain_name, endpoint->aors);
+			aor_name = find_aor_name(&username, &uri->host, endpoint->aors);
 			if (aor_name) {
 				ast_debug(3, "Matched aor '%s' by To username\n", aor_name);
 			}
@@ -908,12 +1066,8 @@ static struct ast_sip_aor *find_registrar_aor(struct pjsip_rx_data *rdata, struc
 			while ((header = pjsip_msg_find_hdr(rdata->msg_info.msg, PJSIP_H_AUTHORIZATION,
 				header ? header->next : NULL))) {
 				if (header && !pj_stricmp2(&header->scheme, "digest")) {
-					username = ast_alloca(header->credential.digest.username.slen + 1);
-					ast_copy_pj_str(username, &header->credential.digest.username, header->credential.digest.username.slen + 1);
-					domain_name = ast_alloca(header->credential.digest.realm.slen + 1);
-					ast_copy_pj_str(domain_name, &header->credential.digest.realm, header->credential.digest.realm.slen + 1);
-
-					aor_name = find_aor_name(username, domain_name, endpoint->aors);
+					aor_name = find_aor_name(&header->credential.digest.username,
+						&header->credential.digest.realm, endpoint->aors);
 					if (aor_name) {
 						ast_debug(3, "Matched aor '%s' by Authentication username\n", aor_name);
 						break;
@@ -935,7 +1089,7 @@ static struct ast_sip_aor *find_registrar_aor(struct pjsip_rx_data *rdata, struc
 		pjsip_endpt_respond_stateless(ast_sip_get_pjsip_endpoint(), rdata, 404, NULL, NULL, NULL);
 		ast_sip_report_req_no_support(endpoint, rdata, "registrar_requested_aor_not_found");
 		ast_log(LOG_WARNING, "AOR '%s' not found for endpoint '%s'\n",
-			username ?: "", ast_sorcery_object_get_id(endpoint));
+			aor_name ?: "", ast_sorcery_object_get_id(endpoint));
 	}
 	ast_free(aor_name);
 	return aor;
@@ -1120,20 +1274,7 @@ static int expire_contact(void *obj, void *arg, int flags)
 	 */
 	ao2_lock(lock);
 	if (ast_tvdiff_ms(ast_tvnow(), contact->expiration_time) > 0) {
-		if (contact->prune_on_boot) {
-			struct contact_transport_monitor *monitor;
-			const char *contact_name = ast_sorcery_object_get_id(contact);
-
-			monitor = ast_alloca(sizeof(*monitor) + 2 + strlen(contact->aor)
-				+ strlen(contact_name));
-			strcpy(monitor->aor_name, contact->aor);/* Safe */
-			monitor->contact_name = monitor->aor_name + strlen(contact->aor) + 1;
-			strcpy(monitor->contact_name, contact_name);/* Safe */
-
-			ast_sip_transport_monitor_unregister_all(register_contact_transport_shutdown_cb,
-				monitor, contact_transport_monitor_matcher);
-		}
-		ast_sip_location_delete_contact(contact);
+		registrar_contact_delete(CONTACT_DELETE_EXPIRE, NULL, contact, contact->aor);
 	}
 	ao2_unlock(lock);
 	ast_named_lock_put(lock);
