@@ -32,35 +32,39 @@
 #include "asterisk/res_pjsip_session.h"
 #include "asterisk/module.h"
 #include "asterisk/causes.h"
+#include "asterisk/threadpool.h"
 
 static void rfc3326_use_reason_header(struct ast_sip_session *session, struct pjsip_rx_data *rdata)
 {
-	const pj_str_t str_reason = { "Reason", 6 };
-	pjsip_generic_string_hdr *header = pjsip_msg_find_hdr_by_name(rdata->msg_info.msg, &str_reason, NULL);
-	char buf[20], *cause, *text;
+	static const pj_str_t str_reason = { "Reason", 6 };
+	pjsip_generic_string_hdr *header;
+	char buf[20];
+	char *cause;
+	char *text;
 	int code;
 
-	if (!header) {
-		return;
+	header = pjsip_msg_find_hdr_by_name(rdata->msg_info.msg, &str_reason, NULL);
+	for (; header;
+		header = pjsip_msg_find_hdr_by_name(rdata->msg_info.msg, &str_reason, header->next)) {
+		ast_copy_pj_str(buf, &header->hvalue, sizeof(buf));
+		cause = ast_skip_blanks(buf);
+
+		if (strncasecmp(cause, "Q.850", 5) || !(cause = strstr(cause, "cause="))) {
+			continue;
+		}
+
+		/* If text is present get rid of it */
+		if ((text = strstr(cause, ";"))) {
+			*text = '\0';
+		}
+
+		if (sscanf(cause, "cause=%30d", &code) != 1) {
+			continue;
+		}
+
+		ast_channel_hangupcause_set(session->channel, code & 0x7f);
+		break;
 	}
-
-	ast_copy_pj_str(buf, &header->hvalue, sizeof(buf));
-	cause = ast_skip_blanks(buf);
-
-	if (strncasecmp(cause, "Q.850", 5) || !(cause = strstr(cause, "cause="))) {
-		return;
-	}
-
-	/* If text is present get rid of it */
-	if ((text = strstr(cause, ";"))) {
-		*text = '\0';
-	}
-
-	if (sscanf(cause, "cause=%30d", &code) != 1) {
-		return;
-	}
-
-	ast_channel_hangupcause_set(session->channel, code & 0x7f);
 }
 
 static int rfc3326_incoming_request(struct ast_sip_session *session, struct pjsip_rx_data *rdata)
@@ -91,19 +95,32 @@ static void rfc3326_add_reason_header(struct ast_sip_session *session, struct pj
 {
 	char buf[20];
 
-	snprintf(buf, sizeof(buf), "Q.850;cause=%i", ast_channel_hangupcause(session->channel) & 0x7f);
-	ast_sip_add_header(tdata, "Reason", buf);
-
 	if (ast_channel_hangupcause(session->channel) == AST_CAUSE_ANSWERED_ELSEWHERE) {
 		ast_sip_add_header(tdata, "Reason", "SIP;cause=200;text=\"Call completed elsewhere\"");
 	}
+
+	if (session->endpoint && session->endpoint->suppress_q850_reason_headers) {
+		ast_debug(1, "A Q.850 '%s'(%i) Reason header was suppresed for endpoint '%s'\n",
+			ast_cause2str((ast_channel_hangupcause(session->channel) & 0x7f)),
+			(ast_channel_hangupcause(session->channel) & 0x7f),
+			ast_sorcery_object_get_id(session->endpoint));
+	} else {
+		snprintf(buf, sizeof(buf), "Q.850;cause=%i", ast_channel_hangupcause(session->channel) & 0x7f);
+		ast_sip_add_header(tdata, "Reason", buf);
+	}
 }
 
 static void rfc3326_outgoing_request(struct ast_sip_session *session, struct pjsip_tx_data *tdata)
 {
-	if ((pjsip_method_cmp(&tdata->msg->line.req.method, &pjsip_bye_method) &&
-	     pjsip_method_cmp(&tdata->msg->line.req.method, &pjsip_cancel_method)) ||
-	    !session->channel) {
+	if ((pjsip_method_cmp(&tdata->msg->line.req.method, &pjsip_bye_method)
+			&& pjsip_method_cmp(&tdata->msg->line.req.method, &pjsip_cancel_method))
+		|| !session->channel
+		/*
+		 * The session->channel has been seen to go away on us between
+		 * checks so we must also be running under the call's serializer
+		 * thread.
+		 */
+		|| session->serializer != ast_threadpool_serializer_get_current()) {
 		return;
 	}
 
@@ -114,7 +131,9 @@ static void rfc3326_outgoing_response(struct ast_sip_session *session, struct pj
 {
 	struct pjsip_status_line status = tdata->msg->line.status;
 
-	if ((status.code < 300) || !session->channel) {
+	if (status.code < 300
+		|| !session->channel
+		|| session->serializer != ast_threadpool_serializer_get_current()) {
 		return;
 	}
 
