@@ -30,7 +30,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 420879 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/module.h"
 #include "asterisk/stasis_app_impl.h"
@@ -74,6 +74,8 @@ struct stasis_app_snoop {
 	unsigned int whisper_active:1;
 	/*! \brief Uniqueid of the channel this snoop is snooping on */
 	char uniqueid[AST_MAX_UNIQUEID];
+	/*! \brief A frame of silence to use when the audiohook returns null */
+	struct ast_frame silence;
 };
 
 /*! \brief Destructor for snoop structure */
@@ -93,6 +95,11 @@ static void snoop_destroy(void *obj)
 		ast_audiohook_destroy(&snoop->whisper);
 	}
 
+	if (snoop->silence.data.ptr) {
+		ast_free(snoop->silence.data.ptr);
+		snoop->silence.data.ptr = NULL;
+	}
+
 	ast_free(snoop->app);
 
 	ast_channel_cleanup(snoop->chan);
@@ -177,13 +184,29 @@ static struct ast_frame *snoop_read(struct ast_channel *chan)
 	}
 
 	/* Only get audio from the spy audiohook if it is active */
-	if (snoop->spy_active) {
-		ast_audiohook_lock(&snoop->spy);
-		frame = ast_audiohook_read_frame(&snoop->spy, snoop->spy_samples, snoop->spy_direction, snoop->spy_format);
-		ast_audiohook_unlock(&snoop->spy);
+	if (!snoop->spy_active) {
+		return &ast_null_frame;
 	}
 
-	return frame ? frame : &ast_null_frame;
+	ast_audiohook_lock(&snoop->spy);
+	if (snoop->spy_direction != AST_AUDIOHOOK_DIRECTION_BOTH) {
+		/*
+		 * When a singular direction is chosen frames are still written to the
+		 * opposing direction's queue. Those frames must be read so the queue
+		 * does not continue to grow, however since they are not needed for the
+		 * selected direction they can be dropped.
+		 */
+		enum ast_audiohook_direction opposing_direction =
+			snoop->spy_direction == AST_AUDIOHOOK_DIRECTION_READ ?
+			AST_AUDIOHOOK_DIRECTION_WRITE : AST_AUDIOHOOK_DIRECTION_READ;
+		ast_frame_dtor(ast_audiohook_read_frame(&snoop->spy, snoop->spy_samples,
+							opposing_direction, snoop->spy_format));
+	}
+
+	frame = ast_audiohook_read_frame(&snoop->spy, snoop->spy_samples, snoop->spy_direction, snoop->spy_format);
+	ast_audiohook_unlock(&snoop->spy);
+
+	return frame ? frame : &snoop->silence;
 }
 
 /*! \brief Callback function for hanging up a Snoop channel */
@@ -369,6 +392,19 @@ struct ast_channel *stasis_app_control_snoop(struct ast_channel *chan,
 
 		snoop->spy_samples = ast_format_get_sample_rate(snoop->spy_format) / (1000 / SNOOP_INTERVAL);
 		snoop->spy_active = 1;
+
+		snoop->silence.frametype = AST_FRAME_VOICE,
+		snoop->silence.datalen = snoop->spy_samples * sizeof(uint16_t),
+		snoop->silence.samples = snoop->spy_samples,
+		snoop->silence.mallocd = 0,
+		snoop->silence.offset = 0,
+		snoop->silence.src = __PRETTY_FUNCTION__,
+		snoop->silence.subclass.format = snoop->spy_format,
+		snoop->silence.data.ptr = ast_calloc(snoop->spy_samples, sizeof(uint16_t));
+		if (!snoop->silence.data.ptr) {
+			ast_hangup(snoop->chan);
+			return NULL;
+		}
 	}
 
 	/* If whispering is enabled set up the audiohook */
@@ -411,4 +447,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, "Stasis applicatio
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis");
+	);
