@@ -36,7 +36,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 419175 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/json.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
@@ -99,7 +99,7 @@ AST_TEST_DEFINE(json_test_false)
 
 	switch (cmd) {
 	case TEST_INIT:
-		info->name = "false";
+		info->name = "type_false";
 		info->category = CATEGORY;
 		info->summary = "Testing fundamental JSON false value.";
 		info->description = "Test JSON abstraction library.";
@@ -124,7 +124,7 @@ AST_TEST_DEFINE(json_test_true)
 
 	switch (cmd) {
 	case TEST_INIT:
-		info->name = "true";
+		info->name = "type_true";
 		info->category = CATEGORY;
 		info->summary = "Testing JSON true value.";
 		info->description = "Test JSON abstraction library.";
@@ -149,7 +149,7 @@ AST_TEST_DEFINE(json_test_bool0)
 
 	switch (cmd) {
 	case TEST_INIT:
-		info->name = "bool0";
+		info->name = "type_bool0";
 		info->category = CATEGORY;
 		info->summary = "Testing JSON boolean function (false).";
 		info->description = "Test JSON abstraction library.";
@@ -176,7 +176,7 @@ AST_TEST_DEFINE(json_test_bool1)
 
 	switch (cmd) {
 	case TEST_INIT:
-		info->name = "bool1";
+		info->name = "type_bool1";
 		info->category = CATEGORY;
 		info->summary = "Testing JSON boolean function (true).";
 		info->description = "Test JSON abstraction library.";
@@ -203,7 +203,7 @@ AST_TEST_DEFINE(json_test_null)
 
 	switch (cmd) {
 	case TEST_INIT:
-		info->name = "null";
+		info->name = "type_null";
 		info->category = CATEGORY;
 		info->summary = "Testing JSON null value.";
 		info->description = "Test JSON abstraction library.";
@@ -255,7 +255,7 @@ AST_TEST_DEFINE(json_test_string)
 
 	switch (cmd) {
 	case TEST_INIT:
-		info->name = "string";
+		info->name = "type_string";
 		info->category = CATEGORY;
 		info->summary = "Basic string tests.";
 		info->description = "Test JSON abstraction library.";
@@ -360,7 +360,7 @@ AST_TEST_DEFINE(json_test_int)
 
 	switch (cmd) {
 	case TEST_INIT:
-		info->name = "int";
+		info->name = "type_int";
 		info->category = CATEGORY;
 		info->summary = "Basic JSON integer tests.";
 		info->description = "Test JSON abstraction library.";
@@ -1597,11 +1597,26 @@ AST_TEST_DEFINE(json_test_clever_circle)
 	return AST_TEST_PASS;
 }
 
+static int test_name_number(const char *name, const char *number)
+{
+	int res;
+	struct ast_json *uut;
+	struct ast_json *expected;
+
+	expected = ast_json_pack("{s: s, s: s}",
+		"name", name ?: "",
+		"number", number ?: "");
+	uut = ast_json_name_number(name, number);
+
+	res = ast_json_equal(expected, uut);
+
+	ast_json_unref(expected);
+	ast_json_unref(uut);
+	return res;
+}
+
 AST_TEST_DEFINE(json_test_name_number)
 {
-	RAII_VAR(struct ast_json *, uut, NULL, ast_json_unref);
-	RAII_VAR(struct ast_json *, expected, NULL, ast_json_unref);
-
 	switch (cmd) {
 	case TEST_INIT:
 		info->name = "name_number";
@@ -1613,15 +1628,10 @@ AST_TEST_DEFINE(json_test_name_number)
 		break;
 	}
 
-	ast_test_validate(test, NULL == ast_json_name_number("name", NULL));
-	ast_test_validate(test, NULL == ast_json_name_number(NULL, "1234"));
-	ast_test_validate(test, NULL == ast_json_name_number(NULL, NULL));
-
-	expected = ast_json_pack("{s: s, s: s}",
-				 "name", "Jenny",
-				 "number", "867-5309");
-	uut = ast_json_name_number("Jenny", "867-5309");
-	ast_test_validate(test, ast_json_equal(expected, uut));
+	ast_test_validate(test, test_name_number("name", NULL));
+	ast_test_validate(test, test_name_number(NULL, "1234"));
+	ast_test_validate(test, test_name_number(NULL, NULL));
+	ast_test_validate(test, test_name_number("Jenny", "867-5309"));
 
 	return AST_TEST_PASS;
 }
@@ -1634,7 +1644,7 @@ AST_TEST_DEFINE(json_test_timeval)
 
 	switch (cmd) {
 	case TEST_INIT:
-		info->name = "timeval";
+		info->name = "type_timeval";
 		info->category = CATEGORY;
 		info->summary = "JSON encoding of timevals.";
 		info->description = "Test JSON abstraction library.";
@@ -1670,20 +1680,26 @@ AST_TEST_DEFINE(json_test_cep)
 		break;
 	}
 
-	expected = ast_json_pack("{s: o, s: o, s: o}",
+	expected = ast_json_pack("{s: o, s: o, s: o, s: o, s: o}",
 				 "context", ast_json_null(),
 				 "exten", ast_json_null(),
-				 "priority", ast_json_null());
-	uut = ast_json_dialplan_cep(NULL, NULL, -1);
+				 "priority", ast_json_null(),
+				 "app_name", ast_json_null(),
+				 "app_data", ast_json_null()
+				 );
+	uut = ast_json_dialplan_cep_app(NULL, NULL, -1, NULL, NULL);
 	ast_test_validate(test, ast_json_equal(expected, uut));
 
 	ast_json_unref(expected);
 	ast_json_unref(uut);
-	expected = ast_json_pack("{s: s, s: s, s: i}",
+	expected = ast_json_pack("{s: s, s: s, s: i, s: s, s: s}",
 				 "context", "main",
 				 "exten", "4321",
-				 "priority", 7);
-	uut = ast_json_dialplan_cep("main", "4321", 7);
+				 "priority", 7,
+				 "app_name", "",
+				 "app_data", ""
+				 );
+	uut = ast_json_dialplan_cep_app("main", "4321", 7, "", "");
 	ast_test_validate(test, ast_json_equal(expected, uut));
 
 	return AST_TEST_PASS;
@@ -1806,5 +1822,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, 0, "JSON testing",
-		.load = load_module,
-		.unload = unload_module);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module
+);
