@@ -31,7 +31,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 420124 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/astobj2.h"
 #include "asterisk/module.h"
@@ -275,9 +275,10 @@ AST_TEST_DEFINE(channel_snapshot_json)
 
 	actual = ast_channel_snapshot_to_json(snapshot, NULL);
 	expected = ast_json_pack("{ s: s, s: s, s: s, s: s,"
-				 "  s: { s: s, s: s, s: i },"
+				 "  s: { s: s, s: s, s: i, s: s, s: s },"
 				 "  s: { s: s, s: s },"
 				 "  s: { s: s, s: s },"
+				 "  s: s"
 				 "  s: o"
 				 "}",
 				 "name", "TEST/name",
@@ -288,12 +289,15 @@ AST_TEST_DEFINE(channel_snapshot_json)
 				 "context", "context",
 				 "exten", "exten",
 				 "priority", 1,
+				 "app_name", "",
+				 "app_data", "",
 				 "caller",
 				 "name", "cid_name",
 				 "number", "cid_num",
 				 "connected",
 				 "name", "",
 				 "number", "",
+				 "language", "en",
 				 "creationtime",
 				 ast_json_timeval(
 					 ast_channel_creationtime(chan), NULL));
@@ -326,6 +330,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, 0, "Stasis Channel Testing",
-		.load = load_module,
-		.unload = unload_module
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module
+);
