@@ -185,9 +185,10 @@ AST_TEST_DEFINE(cache_clear)
 
 	/* Note: there's a few messages between the creation and the clear.
 	 * Wait for all of them... */
-	message_index = stasis_message_sink_wait_for(sink, message_index + 4,
+	message_index = stasis_message_sink_wait_for(sink, message_index + 2,
 		cache_update, __func__, STASIS_SINK_DEFAULT_WAIT);
 	ast_test_validate(test, 0 <= message_index);
+
 	/* Now we should have a cache removal entry */
 	msg = sink->messages[message_index];
 	type = stasis_message_type(msg);
@@ -304,7 +305,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Endpoint stasis-related testing",
+	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis_test",
 	);
