@@ -5,7 +5,7 @@
  *
  * Mark Spencer <markster@digium.com>
  *
- * Updated for Mac OSX CoreAudio 
+ * Updated for Mac OSX CoreAudio
  * by Josh Roberson <josh@asteriasgi.com>
  *
  * See http://www.asterisk.org for more information about
@@ -25,7 +25,7 @@
  *
  * \author Mark Spencer <markster@digium.com>
  *
- * Updated for Mac OSX CoreAudio 
+ * Updated for Mac OSX CoreAudio
  * \arg Josh Roberson <josh@asteriasgi.com>
  *
  * \note Specially written for Malcolm Davenport, but I think I'll use it too
@@ -35,6 +35,15 @@
  *
  */
 
+/*! \li \ref muted.c uses the configuration file \ref muted.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*!
+ * \page muted.conf muted.conf
+ * \verbinclude muted.conf.sample
+ */
+
 /*** MODULEINFO
 	<support_level>extended</support_level>
  ***/
@@ -42,11 +51,11 @@
 #include "asterisk/autoconfig.h"
 
 #ifdef __Darwin__
-#include <CoreAudio/AudioHardware.h> 
+#include <CoreAudio/AudioHardware.h>
 #include <sys/types.h>
 #include <pwd.h>
 #include <sys/stat.h>
-#elif defined(__linux__) || defined(__FreeBSD__) || defined(__GLIBC__)
+#elif defined(__linux__) || defined(__FreeBSD__) || defined(__DragonFly__) || defined(__GLIBC__)
 #include <sys/soundcard.h>
 #endif
 #include <stdio.h>
@@ -108,7 +117,7 @@ static void add_channel(char *tech, char *location)
 		chan->next = channels;
 		channels = chan;
 	}
-	
+
 }
 
 static int load_config(void)
@@ -154,12 +163,12 @@ static int load_config(void)
 					fprintf(stderr, "host needs an argument (the host) at line %d\n", lineno);
 			} else if (!strcasecmp(buf, "user")) {
 				if (val && strlen(val))
-					strncpy(user, val, sizeof(user) - 1);
+					snprintf(user, sizeof(user), "%s", val);
 				else
 					fprintf(stderr, "user needs an argument (the user) at line %d\n", lineno);
 			} else if (!strcasecmp(buf, "pass")) {
 				if (val && strlen(val))
-					strncpy(pass, val, sizeof(pass) - 1);
+					snprintf(pass, sizeof(pass), "%s", val);
 				else
 					fprintf(stderr, "pass needs an argument (the password) at line %d\n", lineno);
 			} else if (!strcasecmp(buf, "smoothfade")) {
@@ -167,7 +176,7 @@ static int load_config(void)
 			} else if (!strcasecmp(buf, "mutelevel")) {
 				if (val && (sscanf(val, "%3d", &x) == 1) && (x > -1) && (x < 101)) {
 					mutelevel = x;
-				} else 
+				} else
 					fprintf(stderr, "mutelevel must be a number from 0 (most muted) to 100 (no mute) at line %d\n", lineno);
 			} else if (!strcasecmp(buf, "channel")) {
 				if (val && strlen(val)) {
@@ -190,7 +199,7 @@ static int load_config(void)
 		fprintf(stderr, "no 'host' specification in config file\n");
 	else if (!strlen(user))
 		fprintf(stderr, "no 'user' specification in config file\n");
-	else if (!channels) 
+	else if (!channels)
 		fprintf(stderr, "no 'channel' specifications in config file\n");
 	else
 		return 0;
@@ -277,7 +286,7 @@ static int login_asterisk(void)
 		fprintf(stderr, "disconnected (1)\n");
 		return -1;
 	}
-	fprintf(astf, 
+	fprintf(astf,
 		"Action: Login\r\n"
 		"Username: %s\r\n"
 		"Secret: %s\r\n\r\n", user, pass);
@@ -295,7 +304,7 @@ static int login_asterisk(void)
 		fprintf(stderr, "disconnected (3)\n");
 		return -1;
 	}
-	fprintf(astf, 
+	fprintf(astf,
 		"Action: Status\r\n\r\n");
 	if (!(welcome = get_line())) {
 		fprintf(stderr, "disconnected (4)\n");
@@ -395,7 +404,7 @@ static int setvol(float vol)
 {
 #ifndef __Darwin__
 	if (ioctl(mixfd, MIXER_WRITE(mixchan), &vol)) {
-#else	
+#else
 	float volumeL = vol;
 	float volumeR = vol;
 	OSStatus err;
@@ -453,7 +462,7 @@ static float mutedlevel(float orig, float level)
 	master = level * master / 100.0;
 	return master;
 #endif
-	
+
 }
 
 static void mute(void)
@@ -470,7 +479,7 @@ static void mute(void)
 	vol = getvol();
 	oldvol = vol;
 	if (smoothfade)
-#ifdef __Darwin__ 
+#ifdef __Darwin__
 		start = mutelevel;
 #else
 		start = 100;
@@ -578,7 +587,7 @@ static void append_sub(struct channel *chan, char *name)
 	struct subchannel *sub;
 	sub = chan->subs;
 	while(sub) {
-		if (!strcasecmp(sub->name, name)) 
+		if (!strcasecmp(sub->name, name))
 			return;
 		sub = sub->next;
 	}
@@ -630,24 +639,29 @@ static int wait_event(void)
 		return -1;
 	}
 	if (!strncasecmp(resp, "Event: ", strlen("Event: "))) {
-		strncpy(event, resp + strlen("Event: "), sizeof(event) - 1);
+		int event_len = -1;
+		int channel_len = -1;
+		int newname_len = -1;
+		int oldname_len = -1;
+
+		event_len = snprintf(event, sizeof(event), "%s", resp + strlen("Event: "));
 		/* Consume the rest of the non-event */
 		while((resp = get_line()) && strlen(resp)) {
 			if (!strncasecmp(resp, "Channel: ", strlen("Channel: ")))
-				strncpy(channel, resp + strlen("Channel: "), sizeof(channel) - 1);
+				channel_len = snprintf(channel, sizeof(channel), "%s", resp + strlen("Channel: "));
 			if (!strncasecmp(resp, "Newname: ", strlen("Newname: ")))
-				strncpy(newname, resp + strlen("Newname: "), sizeof(newname) - 1);
+				newname_len = snprintf(newname, sizeof(newname), "%s", resp + strlen("Newname: "));
 			if (!strncasecmp(resp, "Oldname: ", strlen("Oldname: ")))
-				strncpy(oldname, resp + strlen("Oldname: "), sizeof(oldname) - 1);
+				oldname_len = snprintf(oldname, sizeof(oldname), "%s", resp + strlen("Oldname: "));
 		}
-		if (strlen(channel)) {
-			if (!strcasecmp(event, "Hangup")) 
+		if (channel_len == strlen(channel)) {
+			if (event_len == strlen(event) && !strcasecmp(event, "Hangup"))
 				hangup_chan(channel);
 			else
 				offhook_chan(channel);
 		}
-		if (strlen(newname) && strlen(oldname)) {
-			if (!strcasecmp(event, "Rename")) {
+		if (newname_len == strlen(newname) && oldname_len == strlen(oldname)) {
+			if (event_len == strlen(event) && !strcasecmp(event, "Rename")) {
 				hangup_chan(oldname);
 				offhook_chan(newname);
 			}
@@ -702,7 +716,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 	if (login_asterisk()) {
-#ifndef __Darwin__		
+#ifndef __Darwin__
 		close(mixfd);
 #endif
 		fclose(astf);
